@@ -15,6 +15,8 @@
 #undef HAVE_TM_GMTOFF
 #undef HAVE_KRB4
 #undef NEED_KRB_SENDAUTH_PROTO
+#undef HAVE_HEIMDAL_KRB5
+#undef HAVE_MIT_KRB5
 #undef HAVE_KRB5
 #undef HAVE_NSS
 #undef HAVE_OPENSSL
@@ -23,6 +25,7 @@
 #undef USE_FCNTL
 #undef USE_FLOCK
 #undef ENABLE_NNTP
+#undef ENABLE_IPv6
 #undef HAVE_BROKEN_SPOOL
 #undef ENABLE_PEDANTIC_PGPMIME
 #undef HAVE_KDE_APPLNK
