@@ -234,7 +234,7 @@ struct prop_info {
 	STRING_PROP   (E_CARD_SIMPLE_FIELD_PHONE_BUSINESS_FAX, "business_fax", "facsimileTelephoneNumber"), 
 	E_STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_HOME_FAX,     "home_fax", "homeFacsimileTelephoneNumber"), 
 	E_STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_OTHER,        "other_phone", "otherPhone"), 
-	E_STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_OTHER,        "other_fax", "otherFacsimileTelephoneNumber"), 
+	E_STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_OTHER_FAX,    "other_fax", "otherFacsimileTelephoneNumber"), 
 	STRING_PROP   (E_CARD_SIMPLE_FIELD_PHONE_ISDN,         "isdn", "internationaliSDNNumber"), 
 	STRING_PROP   (E_CARD_SIMPLE_FIELD_PHONE_PAGER,        "pager", "pager"),
 	E_STRING_PROP (E_CARD_SIMPLE_FIELD_PHONE_RADIO,        "radio", "radio"),
@@ -269,10 +269,11 @@ struct prop_info {
 	E_STRING_PROP (E_CARD_SIMPLE_FIELD_MAILER,      "mailer", "mailer"), 
 
 	E_STRING_PROP (E_CARD_SIMPLE_FIELD_FILE_AS,     "file_as", "fileAs"),
-	E_COMPLEX_PROP (E_CARD_SIMPLE_FIELD_CATEGORIES,  "category", "category", category_populate, category_ber, category_compare),
+	E_COMPLEX_PROP (E_CARD_SIMPLE_FIELD_CATEGORIES,  "categories", "category", category_populate, category_ber, category_compare),
 
 	STRING_PROP (E_CARD_SIMPLE_FIELD_CALURI,      "caluri", "calCalURI"),
 	STRING_PROP (E_CARD_SIMPLE_FIELD_FBURL,       "fburl", "calFBURL"),
+	STRING_PROP (E_CARD_SIMPLE_FIELD_ICSCALENDAR, "icscalendar", "icsCalendar"),
 
 /*  	E_CARD_SIMPLE_FIELD_NAME_OR_ORG, */
 
@@ -479,6 +480,22 @@ check_schema_support (PASBackendLDAP *bl)
 
 			ldap_value_free (values);
 		}
+		else {
+			/* the reason for this is so that if the user
+			   ends up authenticating to the ldap server,
+			   we will requery for the subschema values.
+			   This makes it a bit more robust in the face
+			   of draconian acl's that keep subschema
+			   reads from working until the user is
+			   authed. */
+			if (!bl->priv->writable) {
+				g_warning ("subschema read returned nothing before successful auth");
+				bl->priv->evolutionPersonChecked = FALSE;
+			}
+			else {
+				g_warning ("subschema read returned nothing after successful auth");
+			}
+		}
 
 		ldap_msgfree (resp);
 	}
@@ -550,7 +567,7 @@ query_ldap_root_dse (PASBackendLDAP *bl)
 					"(objectclass=*)",
 					attrs, 0, NULL, NULL, &timeout, LDAP_NO_LIMIT, &resp);
 	if (ldap_error != LDAP_SUCCESS) {
-		g_warning ("could not perform query on Root DSE");
+		g_warning ("could not perform query on Root DSE (ldap_error 0x%02x)", ldap_error);
 		return ldap_error;
 	}
 
@@ -586,6 +603,7 @@ query_ldap_root_dse (PASBackendLDAP *bl)
 		values = ldap_get_values (ldap, resp, "schemaNamingContext");
 	}
 	if (values && values[0]) {
+		g_free (bl->priv->schema_dn);
 		bl->priv->schema_dn = g_strdup (values[0]);
 	}
 	else {
@@ -619,34 +637,54 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 	if (NULL != blpriv->ldap) {
 		int ldap_error;
 
-		int protocol_version = LDAP_VERSION3;
-		ldap_error = ldap_set_option (blpriv->ldap, LDAP_OPT_PROTOCOL_VERSION, &protocol_version);
-		if (LDAP_OPT_SUCCESS != ldap_error) {
-			g_warning ("failed to set protocol version to LDAPv3");
-			bl->priv->ldap_v3 = FALSE;
-		}
-		else
-			bl->priv->ldap_v3 = TRUE;
-
-		if (bl->priv->ldap_port == LDAPS_PORT && bl->priv->use_tls == PAS_BACKEND_LDAP_TLS_ALWAYS) {
-			int tls_level = LDAP_OPT_X_TLS_HARD;
-			ldap_set_option (blpriv->ldap, LDAP_OPT_X_TLS, &tls_level);
-		}
-		else if (bl->priv->use_tls) {
-			ldap_error = ldap_start_tls_s (blpriv->ldap, NULL, NULL);
-			if (LDAP_SUCCESS != ldap_error) {
-				if (bl->priv->use_tls == PAS_BACKEND_LDAP_TLS_ALWAYS) {
-					g_message ("TLS not available (fatal version), (ldap_error 0x%02x)", ldap_error);
-					ldap_unbind (blpriv->ldap);
-					blpriv->ldap = NULL;
-					return GNOME_Evolution_Addressbook_BookListener_TLSNotAvailable;
-				}
-				else {
-					g_message ("TLS not available (ldap_error 0x%02x)", ldap_error);
-				}
+		if (bl->priv->use_tls != PAS_BACKEND_LDAP_TLS_NO) {
+			int protocol_version = LDAP_VERSION3;
+			ldap_error = ldap_set_option (blpriv->ldap, LDAP_OPT_PROTOCOL_VERSION, &protocol_version);
+			if (LDAP_OPT_SUCCESS != ldap_error) {
+				g_warning ("failed to set protocol version to LDAPv3");
+				bl->priv->ldap_v3 = FALSE;
 			}
 			else
-				g_message ("TLS active");
+				bl->priv->ldap_v3 = TRUE;
+
+			if (!bl->priv->ldap_v3 && bl->priv->use_tls == PAS_BACKEND_LDAP_TLS_ALWAYS) {
+				g_message ("TLS not available (fatal version), v3 protocol could not be established (ldap_error 0x%02x)", ldap_error);
+				ldap_unbind (blpriv->ldap);
+				blpriv->ldap = NULL;
+				return GNOME_Evolution_Addressbook_BookListener_TLSNotAvailable;
+			}
+
+			if (bl->priv->ldap_port == LDAPS_PORT && bl->priv->use_tls == PAS_BACKEND_LDAP_TLS_ALWAYS) {
+				int tls_level = LDAP_OPT_X_TLS_HARD;
+				ldap_set_option (blpriv->ldap, LDAP_OPT_X_TLS, &tls_level);
+			}
+			else if (bl->priv->use_tls) {
+				ldap_error = ldap_start_tls_s (blpriv->ldap, NULL, NULL);
+				if (LDAP_SUCCESS != ldap_error) {
+					if (bl->priv->use_tls == PAS_BACKEND_LDAP_TLS_ALWAYS) {
+						g_message ("TLS not available (fatal version), (ldap_error 0x%02x)", ldap_error);
+						ldap_unbind (blpriv->ldap);
+						blpriv->ldap = NULL;
+						return GNOME_Evolution_Addressbook_BookListener_TLSNotAvailable;
+					}
+					else {
+						g_message ("TLS not available (ldap_error 0x%02x)", ldap_error);
+					}
+				}
+				else
+					g_message ("TLS active");
+			}
+		}
+
+		/* bind anonymously initially, we'll actually
+		   authenticate the user properly later (in
+		   authenticate_user) if they've selected
+		   authentication */
+		ldap_error = ldap_simple_bind_s (blpriv->ldap, NULL, NULL);
+		if (ldap_error == LDAP_SERVER_DOWN) {
+			/* we only want this to be fatal if the server is down. */
+			g_warning ("failed to bind anonymously while connecting (ldap_error 0x%02x)", ldap_error);
+			return GNOME_Evolution_Addressbook_BookListener_RepositoryOffline;
 		}
 
 		ldap_error = query_ldap_root_dse (bl);
@@ -654,7 +692,11 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 		   connect(), so any tcpip problems will show up
 		   here */
 
-		if (LDAP_SUCCESS == ldap_error) {
+		/* we can't just check for LDAP_SUCCESS here since in
+		   older servers (namely openldap1.x servers), there's
+		   not a root DSE at all, so the query will fail with
+		   LDAP_NO_SUCH_OBJECT. */
+		if (ldap_error == LDAP_SUCCESS || LDAP_NAME_ERROR (ldap_error)) {
 			blpriv->connected = TRUE;
 
 			/* check to see if evolutionPerson is supported, if we can (me
@@ -665,6 +707,8 @@ pas_backend_ldap_connect (PASBackendLDAP *bl)
 
 			return GNOME_Evolution_Addressbook_BookListener_Success;
 		}
+		else
+			g_warning ("Failed to perform root dse query anonymously, (ldap_error 0x%02x)", ldap_error);
 	}
 
 	g_warning ("pas_backend_ldap_connect failed for "
@@ -3133,8 +3177,20 @@ pas_backend_ldap_process_authenticate_user (PASBackend *backend,
 
 	bl->priv->writable = (ldap_error == LDAP_SUCCESS);
 
-	if (!bl->priv->evolutionPersonChecked)
-		check_schema_support (bl);
+	/* if the bind was successful we force a requery on the root
+	   dse since some ldap servers are set up such that they don't
+	   report anything (including the schema DN) until the user is
+	   authenticated */
+	if (!bl->priv->evolutionPersonChecked && ldap_error == LDAP_SUCCESS) {
+		ldap_error = query_ldap_root_dse (bl);
+
+		if (LDAP_SUCCESS == ldap_error) {
+			if (!bl->priv->evolutionPersonChecked)
+				check_schema_support (bl);
+		}
+		else
+			g_warning ("Failed to perform root dse query after authenticating, (ldap_error 0x%02x)", ldap_error);
+	}
 
 	pas_book_report_writable (book, bl->priv->writable);
 }
@@ -3268,7 +3324,7 @@ pas_backend_ldap_load_uri (PASBackend             *backend,
 				else if (!strncmp (value, "whenever_possible", 3)) {
 					bl->priv->use_tls = PAS_BACKEND_LDAP_TLS_WHEN_POSSIBLE;
 				}
-				else {
+				else if (strncmp (value, "never", 5)) {
 					g_warning ("unhandled value for use_tls, not using it");
 				}
 			}
