@@ -114,22 +114,6 @@ e_select_names_class_init (ESelectNamesClass *klass)
 	object_class->destroy = e_select_names_destroy;
 }
 
-#define SPEC "<ETableSpecification no-headers=\"true\" cursor-mode=\"line\"> \
-  <ETableColumn model_col= \"%d\" _title=\"Name\"          expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"       compare=\"string\" search=\"string\"/> \
-	<ETableState>                   			       \
-		<column source=\"0\"/>     			       \
-	        <grouping> <leaf column=\"0\" ascending=\"true\"/> </grouping> \
-	</ETableState>                  			       \
-</ETableSpecification>"
-
-#define SPEC2 "<ETableSpecification no-headers=\"true\" cursor-mode=\"line\"> \
-  <ETableColumn model_col= \"0\" _title=\"Name\"          expansion=\"1.0\" minimum_width=\"20\" resizable=\"true\" cell=\"string\"       compare=\"string\" search=\"string\"/> \
-	<ETableState>                   			       \
-		<column source=\"0\"/>     			       \
-        	<grouping> <leaf column=\"0\" ascending=\"true\"/> </grouping> \
-	</ETableState>                  			       \
-</ETableSpecification>"
-
 GtkWidget *e_addressbook_create_ebook_table(char *name, char *string1, char *string2, int num1, int num2);
 GtkWidget *e_addressbook_create_folder_selector(char *name, char *string1, char *string2, int num1, int num2);
 
@@ -332,9 +316,10 @@ e_addressbook_create_ebook_table(char *name, char *string1, char *string2, int n
 				       esn_free_duped_key_fn,
 				       model);
 
-	spec = g_strdup_printf(SPEC, E_CARD_SIMPLE_FIELD_NAME_OR_ORG);
-	table = e_table_scrolled_new (without, NULL, spec, NULL);
-	g_free(spec);
+	table = e_table_scrolled_new_from_spec_file (without,
+						     NULL,
+						     EVOLUTION_ETSPECDIR "/e-select-names.etspec",
+						     NULL);
 
 	gtk_object_set_data(GTK_OBJECT(table), "adapter", adapter);
 	gtk_object_set_data(GTK_OBJECT(table), "without", without);
@@ -353,7 +338,12 @@ static void
 folder_selected (EvolutionFolderSelectorButton *button, GNOME_Evolution_Folder *folder,
 		 ESelectNames *e_select_names)
 {
+	Bonobo_ConfigDatabase db;
+
 	addressbook_model_set_uri(e_select_names, e_select_names->model, folder->physicalUri);
+
+	db = addressbook_config_database (NULL);
+	bonobo_config_set_string (db, "/Addressbook/select_names_uri", folder->physicalUri, NULL);
 }
 
 static void
@@ -400,10 +390,16 @@ update_query (GtkWidget *widget, ESelectNames *e_select_names)
 static void
 status_message (EAddressbookModel *model, const gchar *message, ESelectNames *e_select_names)
 {
-	if (message == NULL)
+	if (message == NULL) {
+		if (e_select_names->status_image)
+			gtk_widget_hide (e_select_names->status_image);
 		gtk_label_set_text (GTK_LABEL (e_select_names->status_message), "");
-	else
+	}
+	else {
+		if (e_select_names->status_image)
+			gtk_widget_show (e_select_names->status_image);
 		gtk_label_set_text (GTK_LABEL (e_select_names->status_message), message);
+	}
 }
 
 static void
@@ -483,8 +479,7 @@ e_select_names_init (ESelectNames *e_select_names)
 	GtkWidget *widget, *button;
 	const char *selector_types[] = { "contacts/*", NULL };
 	char *filename;
-	char *uri;
-	char *contacts_uri, *contacts_path;
+	char *contacts_uri;
 	Bonobo_ConfigDatabase db;
 
 	db = addressbook_config_database (NULL);
@@ -528,6 +523,8 @@ e_select_names_init (ESelectNames *e_select_names)
 		gtk_signal_connect(GTK_OBJECT(e_select_names->status_message), "destroy",
 				   GTK_SIGNAL_FUNC(clear_widget), &e_select_names->status_message);
 	}
+	e_select_names->status_image = glade_xml_get_widget (gui, "status-image");
+	gtk_widget_hide (e_select_names->status_image);
 
 	e_select_names->categories = glade_xml_get_widget (gui, "custom-categories");
 	if (e_select_names->categories && !E_IS_CATEGORIES_MASTER_LIST_OPTION_MENU (e_select_names->categories))
@@ -556,25 +553,24 @@ e_select_names_init (ESelectNames *e_select_names)
 		gtk_signal_connect(GTK_OBJECT(button), "clicked",
 				   GTK_SIGNAL_FUNC(update_query), e_select_names);
 
-	contacts_path = bonobo_config_get_string_with_default (db, "/DefaultFolders/contacts_path",
-							       "evolution:/local/Contacts",
-							       NULL);
-
-	filename = gnome_util_prepend_user_home("evolution/local/Contacts");
-	uri = g_strdup_printf("file://%s", filename);
-
-	contacts_uri = bonobo_config_get_string_with_default (db, "/DefaultFolders/contacts_uri",
-							      uri,
-							      NULL);
-
-	g_free (filename);
-	g_free (uri);
+	contacts_uri = bonobo_config_get_string_with_default (db, "/Addressbook/select_names_uri",
+							      NULL, NULL);
+	if (!contacts_uri) {
+		contacts_uri = bonobo_config_get_string_with_default (db, "/DefaultFolders/contacts_uri",
+								      NULL,
+								      NULL);
+	}
+	if (!contacts_uri) {
+		filename = gnome_util_prepend_user_home("evolution/local/Contacts");
+		contacts_uri = g_strdup_printf("file://%s", filename);
+		g_free (filename);
+	}
 
 	button = glade_xml_get_widget (gui, "folder-selector");
 	evolution_folder_selector_button_construct (EVOLUTION_FOLDER_SELECTOR_BUTTON (button),
 						    global_shell_client,
 						    _("Find contact in"),
-						    contacts_path,
+						    contacts_uri,
 						    selector_types);
 	if (button && EVOLUTION_IS_FOLDER_SELECTOR_BUTTON (button))
 		gtk_signal_connect(GTK_OBJECT(button), "selected",
@@ -589,7 +585,6 @@ e_select_names_init (ESelectNames *e_select_names)
 	addressbook_model_set_uri(e_select_names, e_select_names->model, contacts_uri);
 
 	g_free (contacts_uri);
-	g_free (contacts_path);
 }
 
 static void e_select_names_child_free(char *key, ESelectNamesChild *child, ESelectNames *e_select_names)
