@@ -9,9 +9,8 @@
  * Copyright 2000, Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,11 +37,17 @@
 #include <gal/e-table/e-cell-toggle.h>
 #include <gal/e-table/e-cell-text.h>
 #include <gal/e-table/e-cell-combo.h>
+#include <gal/util/e-unicode-i18n.h>
 #include <gal/widgets/e-popup-menu.h>
+#include <e-util/e-dialog-utils.h>
 #include <widgets/misc/e-cell-date-edit.h>
+#include <widgets/misc/e-cell-percent.h>
+#include "e-comp-editor-registry.h"
 #include "e-calendar-table.h"
+#include "e-cell-date-edit-text.h"
 #include "calendar-config.h"
 #include "calendar-model.h"
+#include "print.h"
 #include "dialogs/delete-comp.h"
 #include "dialogs/task-editor.h"
 
@@ -54,6 +59,7 @@
 
 #include "art/check-filled.xpm"
 
+extern ECompEditorRegistry *comp_editor_registry;
 
 static void e_calendar_table_class_init		(ECalendarTableClass *class);
 static void e_calendar_table_init		(ECalendarTable	*cal_table);
@@ -71,12 +77,20 @@ static gint e_calendar_table_on_right_click	(ETable		*table,
 						 ECalendarTable *cal_table);
 static void e_calendar_table_on_open_task	(GtkWidget	*menuitem,
 						 gpointer	 data);
+static void e_calendar_table_on_save_as	        (GtkWidget	*menuitem,
+						 gpointer	 data);
+static void e_calendar_table_on_print_task      (GtkWidget	*menuitem,
+						 gpointer	 data);
 static void e_calendar_table_on_cut             (GtkWidget      *menuitem,
 						 gpointer        data);
 static void e_calendar_table_on_copy            (GtkWidget      *menuitem,
 						 gpointer        data);
 static void e_calendar_table_on_paste           (GtkWidget      *menuitem,
 						 gpointer        data);
+static void e_calendar_table_on_assign          (GtkWidget      *menuitem,
+						 gpointer        data);
+static void e_calendar_table_on_forward         (GtkWidget      *menuitem,
+						 gpointer        data);
 static gint e_calendar_table_on_key_press	(ETable		*table,
 						 gint		 row,
 						 gint		 col,
@@ -99,6 +113,7 @@ static void invisible_destroyed                 (GtkWidget *invisible,
 						 ECalendarTable *cal_table);
 static struct tm e_calendar_table_get_current_time (ECellDateEdit *ecde,
 						    gpointer data);
+static void mark_row_complete_cb (int model_row, gpointer data);
 
 
 /* The icons to represent the task. */
@@ -186,6 +201,12 @@ compare_priorities (int *a, int *b)
 
 /* Comparison function for the task-sort column.  Sorts by due date and then by
  * priority.
+ *
+ * FIXME: Does this ever get called?? It doesn't seem to.
+ * I specified that the table should be sorted by this column, but it still
+ * never calls this function.
+ * Also, this assumes it is passed pointers to CalComponents, but I think it
+ * may just be passed pointers to the 2 cell values.
  */
 static gint
 task_compare_cb (gconstpointer a, gconstpointer b)
@@ -206,7 +227,8 @@ task_compare_cb (gconstpointer a, gconstpointer b)
 	if (due_a.value && due_b.value) {
 		int v;
 
-		/* FIXME: TIMEZONES. */
+		/* FIXME: TIMEZONES. But currently we have no way to get the
+		   CalClient, so we can't get the timezone. */
 		v = icaltime_compare (*due_a.value, *due_b.value);
 
 		if (v == 0)
@@ -232,6 +254,74 @@ task_compare_cb (gconstpointer a, gconstpointer b)
 	return retval;
 }
 
+static gint
+date_compare_cb (gconstpointer a, gconstpointer b)
+{
+	ECellDateEditValue *dv1 = (ECellDateEditValue *) a;
+	ECellDateEditValue *dv2 = (ECellDateEditValue *) b;
+	struct icaltimetype tt;
+
+	/* First check if either is NULL. NULL dates sort last. */
+	if (!dv1 || !dv2) {
+		if (dv1 == dv2)
+			return 0;
+		else if (dv1)
+			return -1;
+		else
+			return 1;
+	}
+
+	/* Copy the 2nd value and convert it to the same timezone as the
+	   first. */
+	tt = dv2->tt;
+
+	icaltimezone_convert_time (&tt, dv2->zone, dv1->zone);
+
+	/* Now we can compare them. */
+
+	return icaltime_compare (dv1->tt, tt);
+}
+
+static gint
+percent_compare_cb (gconstpointer a, gconstpointer b)
+{
+	int percent1 = GPOINTER_TO_INT (a);
+	int percent2 = GPOINTER_TO_INT (b);
+	int retval;
+
+	if (percent1 > percent2)
+		retval = 1;
+	else if (percent1 < percent2)
+		retval = -1;
+	else
+		retval = 0;
+
+	return retval;
+}
+
+static gint
+priority_compare_cb (gconstpointer a, gconstpointer b)
+{
+	int priority1, priority2;
+
+	priority1 = cal_util_priority_from_string ((const char*) a);
+	priority2 = cal_util_priority_from_string ((const char*) b);
+
+	/* We change undefined priorities so they appear after 'Low'. */
+	if (priority1 <= 0)
+		priority1 = 10;
+	if (priority2 <= 0)
+		priority2 = 10;
+
+	/* We'll just use the ordering of the priority values. */
+	if (priority1 < priority2)
+		return -1;
+	else if (priority1 > priority2)
+		return 1;
+	else
+		return 0;
+}
+
 static void
 e_calendar_table_init (ECalendarTable *cal_table)
 {
@@ -267,7 +357,7 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	/*
 	 * Date fields.
 	 */
-	cell = e_cell_text_new (NULL, GTK_JUSTIFY_LEFT);
+	cell = e_cell_date_edit_text_new (NULL, GTK_JUSTIFY_LEFT);
 	gtk_object_set (GTK_OBJECT (cell),
 			"strikeout_column", CAL_COMPONENT_FIELD_COMPLETE,
 			"bold_column", CAL_COMPONENT_FIELD_OVERDUE,
@@ -303,10 +393,9 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	gtk_object_unref (GTK_OBJECT (cell));
 
 	strings = NULL;
-	strings = g_list_append (strings, _("None"));
-	strings = g_list_append (strings, _("Public"));
-	strings = g_list_append (strings, _("Private"));
-	strings = g_list_append (strings, _("Confidential"));
+	strings = g_list_append (strings, (char*) U_("Public"));
+	strings = g_list_append (strings, (char*) U_("Private"));
+	strings = g_list_append (strings, (char*) U_("Confidential"));
 	e_cell_combo_set_popdown_strings (E_CELL_COMBO (popup_cell),
 					  strings);
 
@@ -326,17 +415,17 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	gtk_object_unref (GTK_OBJECT (cell));
 
 	strings = NULL;
-	strings = g_list_append (strings, _("High"));
-	strings = g_list_append (strings, _("Normal"));
-	strings = g_list_append (strings, _("Low"));
-	strings = g_list_append (strings, _("Undefined"));
+	strings = g_list_append (strings, (char*) U_("High"));
+	strings = g_list_append (strings, (char*) U_("Normal"));
+	strings = g_list_append (strings, (char*) U_("Low"));
+	strings = g_list_append (strings, (char*) U_("Undefined"));
 	e_cell_combo_set_popdown_strings (E_CELL_COMBO (popup_cell),
 					  strings);
 
 	e_table_extras_add_cell (extras, "priority", popup_cell);
 
 	/* Percent field. */
-	cell = e_cell_text_new (NULL, GTK_JUSTIFY_LEFT);
+	cell = e_cell_percent_new (NULL, GTK_JUSTIFY_LEFT);
 	gtk_object_set (GTK_OBJECT (cell),
 			"strikeout_column", CAL_COMPONENT_FIELD_COMPLETE,
 			"bold_column", CAL_COMPONENT_FIELD_OVERDUE,
@@ -348,17 +437,17 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	gtk_object_unref (GTK_OBJECT (cell));
 
 	strings = NULL;
-	strings = g_list_append (strings, _("0%"));
-	strings = g_list_append (strings, _("10%"));
-	strings = g_list_append (strings, _("20%"));
-	strings = g_list_append (strings, _("30%"));
-	strings = g_list_append (strings, _("40%"));
-	strings = g_list_append (strings, _("50%"));
-	strings = g_list_append (strings, _("60%"));
-	strings = g_list_append (strings, _("70%"));
-	strings = g_list_append (strings, _("80%"));
-	strings = g_list_append (strings, _("90%"));
-	strings = g_list_append (strings, _("100%"));
+	strings = g_list_append (strings, (char*) U_("0%"));
+	strings = g_list_append (strings, (char*) U_("10%"));
+	strings = g_list_append (strings, (char*) U_("20%"));
+	strings = g_list_append (strings, (char*) U_("30%"));
+	strings = g_list_append (strings, (char*) U_("40%"));
+	strings = g_list_append (strings, (char*) U_("50%"));
+	strings = g_list_append (strings, (char*) U_("60%"));
+	strings = g_list_append (strings, (char*) U_("70%"));
+	strings = g_list_append (strings, (char*) U_("80%"));
+	strings = g_list_append (strings, (char*) U_("90%"));
+	strings = g_list_append (strings, (char*) U_("100%"));
 	e_cell_combo_set_popdown_strings (E_CELL_COMBO (popup_cell),
 					  strings);
 
@@ -378,9 +467,8 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	gtk_object_unref (GTK_OBJECT (cell));
 
 	strings = NULL;
-	strings = g_list_append (strings, _("None"));
-	strings = g_list_append (strings, _("Opaque"));
-	strings = g_list_append (strings, _("Transparent"));
+	strings = g_list_append (strings, (char*) U_("Free"));
+	strings = g_list_append (strings, (char*) U_("Busy"));
 	e_cell_combo_set_popdown_strings (E_CELL_COMBO (popup_cell),
 					  strings);
 
@@ -400,10 +488,10 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	gtk_object_unref (GTK_OBJECT (cell));
 
 	strings = NULL;
-	strings = g_list_append (strings, _("Not Started"));
-	strings = g_list_append (strings, _("In Progress"));
-	strings = g_list_append (strings, _("Completed"));
-	strings = g_list_append (strings, _("Cancelled"));
+	strings = g_list_append (strings, (char*) U_("Not Started"));
+	strings = g_list_append (strings, (char*) U_("In Progress"));
+	strings = g_list_append (strings, (char*) U_("Completed"));
+	strings = g_list_append (strings, (char*) U_("Cancelled"));
 	e_cell_combo_set_popdown_strings (E_CELL_COMBO (popup_cell),
 					  strings);
 
@@ -415,6 +503,13 @@ e_calendar_table_init (ECalendarTable *cal_table)
 	 */
 	e_table_extras_add_compare (extras, "task-sort", task_compare_cb);
 
+	e_table_extras_add_compare (extras, "date-compare",
+				    date_compare_cb);
+	e_table_extras_add_compare (extras, "percent-compare",
+				    percent_compare_cb);
+	e_table_extras_add_compare (extras, "priority-compare",
+				    priority_compare_cb);
+
 	/* Create pixmaps */
 
 	if (!icon_pixbufs[0])
@@ -555,6 +650,24 @@ e_calendar_table_get_table (ECalendarTable *cal_table)
 	return e_table_scrolled_get_table (E_TABLE_SCROLLED (cal_table->etable));
 }
 
+/**
+ * e_calendar_table_complete_selected:
+ * @cal_table: A calendar table
+ * 
+ * Marks the selected items as completed
+ **/
+void
+e_calendar_table_complete_selected (ECalendarTable *cal_table)
+{
+	ETable *etable;
+
+	g_return_if_fail (cal_table != NULL);
+	g_return_if_fail (E_IS_CALENDAR_TABLE (cal_table));
+
+	etable = e_table_scrolled_get_table (E_TABLE_SCROLLED (cal_table->etable));
+	e_table_selected_row_foreach (etable, mark_row_complete_cb, cal_table);
+}
+
 /* Used from e_table_selected_row_foreach(); puts the selected row number in an
  * int pointed to by the closure data.
  */
@@ -577,7 +690,8 @@ get_selected_comp (ECalendarTable *cal_table)
 	int row;
 
 	etable = e_table_scrolled_get_table (E_TABLE_SCROLLED (cal_table->etable));
-	g_assert (e_table_selected_count (etable) == 1);
+	if (e_table_selected_count (etable) != 1)
+		return NULL;
 
 	row = -1;
 	e_table_selected_row_foreach (etable,
@@ -635,6 +749,9 @@ delete_selected_components (ECalendarTable *cal_table)
 
 	client = calendar_model_get_cal_client (cal_table->model);
 
+	calendar_model_set_status_message (e_calendar_table_get_model (cal_table),
+					   _("Deleting selected objects"));
+
 	for (l = uids; l; l = l->next) {
 		const char *uid;
 
@@ -646,6 +763,8 @@ delete_selected_components (ECalendarTable *cal_table)
 		cal_client_remove_object (client, uid);
 	}
 
+	calendar_model_set_status_message (e_calendar_table_get_model (cal_table), NULL);
+
 	g_slist_free (uids);
 }
 
@@ -668,7 +787,8 @@ e_calendar_table_delete_selected (ECalendarTable *cal_table)
 	etable = e_table_scrolled_get_table (E_TABLE_SCROLLED (cal_table->etable));
 
 	n_selected = e_table_selected_count (etable);
-	g_assert (n_selected > 0);
+	if (n_selected <= 0)
+		return;
 
 	if (n_selected == 1)
 		comp = get_selected_comp (cal_table);
@@ -677,7 +797,8 @@ e_calendar_table_delete_selected (ECalendarTable *cal_table)
 
 	/* FIXME: this may be something other than a TODO component */
 
-	if (delete_component_dialog (comp, n_selected, CAL_COMPONENT_TODO, GTK_WIDGET (cal_table)))
+	if (delete_component_dialog (comp, FALSE, n_selected, CAL_COMPONENT_TODO,
+				     GTK_WIDGET (cal_table)))
 		delete_selected_components (cal_table);
 }
 
@@ -713,11 +834,17 @@ copy_row_cb (int model_row, gpointer data)
 	if (!comp)
 		return;
 
+	/* add timezones to the VCALENDAR component */
+	cal_util_add_timezones_from_component (cal_table->tmp_vcal, comp);
+
 	/* add the new component to the VCALENDAR component */
 	comp_str = cal_component_get_as_string (comp);
 	child = icalparser_parse_string (comp_str);
-	if (child)
-		icalcomponent_add_component (cal_table->tmp_vcal, child);
+	if (child) {
+		icalcomponent_add_component (cal_table->tmp_vcal,
+					     icalcomponent_new_clone (child));
+		icalcomponent_free (child);
+	}
 
 	g_free (comp_str);
 }
@@ -774,14 +901,25 @@ e_calendar_table_paste_clipboard (ECalendarTable *cal_table)
 
 /* Opens a task in the task editor */
 static void
-open_task (ECalendarTable *cal_table, CalComponent *comp)
+open_task (ECalendarTable *cal_table, CalComponent *comp, gboolean assign)
 {
-	TaskEditor *tedit;
+	CompEditor *tedit;
+	const char *uid;
+	
+	cal_component_get_uid (comp, &uid);
 
-	tedit = task_editor_new ();
-	comp_editor_set_cal_client (COMP_EDITOR (tedit), calendar_model_get_cal_client (cal_table->model));
-	comp_editor_edit_comp (COMP_EDITOR (tedit), comp);
-	comp_editor_focus (COMP_EDITOR (tedit));
+	tedit = e_comp_editor_registry_find (comp_editor_registry, uid);
+	if (tedit == NULL) {
+		tedit = COMP_EDITOR (task_editor_new (calendar_model_get_cal_client (cal_table->model)));
+
+		comp_editor_edit_comp (tedit, comp);
+		if (assign)
+			task_editor_show_assignment (TASK_EDITOR (tedit));
+		
+		e_comp_editor_registry_add (comp_editor_registry, tedit, FALSE);
+	}
+	
+	comp_editor_focus (tedit);
 }
 
 /* Opens the task in the specified row */
@@ -791,7 +929,7 @@ open_task_by_row (ECalendarTable *cal_table, int row)
 	CalComponent *comp;
 
 	comp = calendar_model_get_component (cal_table->model, row);
-	open_task (cal_table, comp);
+	open_task (cal_table, comp, FALSE);
 }
 
 static void
@@ -841,34 +979,35 @@ delete_cb (GtkWidget *menuitem, gpointer data)
 enum {
 	MASK_SINGLE	= 1 << 0,	/* For commands that work on 1 task. */
 	MASK_MULTIPLE	= 1 << 1,	/* For commands for multiple tasks. */
+	MASK_EDITABLE   = 1 << 2,       /* For commands disabled in read-only folders */
+	MASK_ASSIGNABLE = 1 << 3        /* For non-task assignable backends */
 };
 
 
 static EPopupMenu tasks_popup_menu [] = {
-	{ N_("_Open"), NULL,
-	  e_calendar_table_on_open_task, NULL, MASK_SINGLE },
-	{ "", NULL, NULL, NULL, MASK_SINGLE },
+	E_POPUP_ITEM (N_("_Open"), e_calendar_table_on_open_task, MASK_SINGLE),
+	E_POPUP_ITEM (N_("_Save as..."), e_calendar_table_on_save_as, MASK_SINGLE),
+	E_POPUP_ITEM (N_("_Print..."), e_calendar_table_on_print_task, MASK_SINGLE),
 
-	{ N_("C_ut"), NULL,
-	  e_calendar_table_on_cut, NULL, 0 },
-	{ N_("_Copy"), NULL,
-	  e_calendar_table_on_copy, NULL, 0 },
-	{ N_("_Paste"), NULL,
-	  e_calendar_table_on_paste, NULL, 0 },
+	E_POPUP_SEPARATOR,
+	
+	E_POPUP_ITEM (N_("C_ut"), e_calendar_table_on_cut, MASK_EDITABLE),
+	E_POPUP_ITEM (N_("_Copy"), e_calendar_table_on_copy, 0),
+	E_POPUP_ITEM (N_("_Paste"), e_calendar_table_on_paste, MASK_EDITABLE),
 
-	{ "", NULL, NULL, NULL, 0 },
+	E_POPUP_SEPARATOR,
 
-	{ N_("_Mark as Complete"), NULL,
-	  mark_as_complete_cb, NULL, MASK_SINGLE },
-	{ N_("_Delete this Task"), NULL,
-	  delete_cb, NULL, MASK_SINGLE },
+	E_POPUP_ITEM (N_("_Assign Task"), e_calendar_table_on_assign, MASK_SINGLE | MASK_EDITABLE | MASK_ASSIGNABLE),
+	E_POPUP_ITEM (N_("_Forward as iCalendar"), e_calendar_table_on_forward, MASK_SINGLE),
+	E_POPUP_ITEM (N_("_Mark as Complete"), mark_as_complete_cb, MASK_SINGLE | MASK_EDITABLE),
+	E_POPUP_ITEM (N_("_Mark Selected Tasks as Complete"), mark_as_complete_cb, MASK_MULTIPLE | MASK_EDITABLE),
+	
+	E_POPUP_SEPARATOR,
 
-	{ N_("_Mark Tasks as Complete"), NULL,
-	  mark_as_complete_cb, NULL, MASK_MULTIPLE },
-	{ N_("_Delete Selected Tasks"), NULL,
-	  delete_cb, NULL, MASK_MULTIPLE },
+	E_POPUP_ITEM (N_("_Delete"), delete_cb, MASK_SINGLE | MASK_EDITABLE),
+	E_POPUP_ITEM (N_("_Delete Selected Tasks"), delete_cb, MASK_MULTIPLE | MASK_EDITABLE),
 
-	{ NULL, NULL, NULL, NULL, 0 }
+	E_POPUP_TERMINATOR
 };
 
 static gint
@@ -883,13 +1022,20 @@ e_calendar_table_on_right_click (ETable *table,
 	int disable_mask = 0;
 
 	n_selected = e_table_selected_count (table);
-	g_assert (n_selected > 0);
+	if (n_selected <= 0)
+		return TRUE;
 
 	if (n_selected == 1)
 		hide_mask = MASK_MULTIPLE;
 	else
 		hide_mask = MASK_SINGLE;
 
+	if (cal_client_is_read_only (calendar_model_get_cal_client (e_calendar_table_get_model (cal_table))))
+		disable_mask |= MASK_EDITABLE;
+
+	if (cal_client_get_static_capability (calendar_model_get_cal_client (e_calendar_table_get_model (cal_table)), "no-task-assignment"))
+		disable_mask |= MASK_ASSIGNABLE;
+
 	e_popup_menu_run (tasks_popup_menu, (GdkEvent *) event,
 			  disable_mask, hide_mask, cal_table);
 
@@ -907,7 +1053,63 @@ e_calendar_table_on_open_task (GtkWidget *menuitem,
 	cal_table = E_CALENDAR_TABLE (data);
 
 	comp = get_selected_comp (cal_table);
-	open_task (cal_table, comp);
+	if (comp)
+		open_task (cal_table, comp, FALSE);
+}
+
+static void
+e_calendar_table_on_save_as (GtkWidget *widget, gpointer data)
+{
+	ECalendarTable *cal_table;
+	CalClient *client;
+	CalComponent *comp;
+	char *filename;
+	char *ical_string;
+	FILE *file;
+
+	cal_table = E_CALENDAR_TABLE (data);
+
+	client = calendar_model_get_cal_client (cal_table->model);
+	comp = get_selected_comp (cal_table);
+	if (comp == NULL)
+		return;
+	
+	filename = e_file_dialog_save (_("Save as..."));
+	if (filename == NULL)
+		return;
+	
+	ical_string = cal_client_get_component_as_string (client, comp);
+	if (ical_string == NULL) {
+		g_warning ("Couldn't convert item to a string");
+		return;
+	}
+	
+	file = fopen (filename, "w");
+	if (file == NULL) {
+		g_warning ("Couldn't save item");
+		return;
+	}
+	
+	fprintf (file, ical_string);
+	g_free (ical_string);
+	fclose (file);
+}
+
+static void
+e_calendar_table_on_print_task (GtkWidget *widget, gpointer data)
+{
+	ECalendarTable *cal_table;
+	CalClient *client;
+	CalComponent *comp;
+
+	cal_table = E_CALENDAR_TABLE (data);
+
+	client = calendar_model_get_cal_client (cal_table->model);
+	comp = get_selected_comp (cal_table);
+	if (comp == NULL)
+		return;
+	
+	print_comp (comp, client, FALSE);
 }
 
 static void
@@ -937,6 +1139,35 @@ e_calendar_table_on_paste (GtkWidget *menuitem, gpointer data)
 	e_calendar_table_paste_clipboard (cal_table);
 }
 
+static void
+e_calendar_table_on_assign (GtkWidget *widget, gpointer data)
+{
+	ECalendarTable *cal_table;
+	CalComponent *comp;
+
+	cal_table = E_CALENDAR_TABLE (data);
+
+	comp = get_selected_comp (cal_table);
+	if (comp)
+		open_task (cal_table, comp, TRUE);
+}
+
+static void
+e_calendar_table_on_forward (GtkWidget *widget, gpointer data)
+{
+	ECalendarTable *cal_table;
+	CalClient *client;
+	CalComponent *comp;
+
+	cal_table = E_CALENDAR_TABLE (data);
+
+	client = calendar_model_get_cal_client (cal_table->model);
+	comp = get_selected_comp (cal_table);
+	if (comp)
+		itip_send_comp (CAL_COMPONENT_METHOD_PUBLISH, comp,
+				client, NULL);
+}
+
 static gint
 e_calendar_table_on_key_press (ETable *table,
 			       gint row,
@@ -983,6 +1214,7 @@ e_calendar_table_save_state (ECalendarTable	*cal_table,
 static void
 invisible_destroyed (GtkWidget *invisible, ECalendarTable *cal_table)
 {
+	gtk_object_unref (GTK_OBJECT (cal_table->invisible));
 	cal_table->invisible = NULL;
 }
 
@@ -1036,7 +1268,7 @@ selection_received (GtkWidget *invisible,
 	icalcomp = icalparser_parse_string ((const char *) comp_str);
 	if (!icalcomp)
 		return;
-	
+
 	/* check the type of the component */
 	kind = icalcomponent_isa (icalcomp);
 	if (kind != ICAL_VCALENDAR_COMPONENT &&
@@ -1046,13 +1278,17 @@ selection_received (GtkWidget *invisible,
 		return;
 	}
 
+	calendar_model_set_status_message (e_calendar_table_get_model (cal_table),
+					   _("Updating objects"));
+
 	if (kind == ICAL_VCALENDAR_COMPONENT) {
-		int num_found = 0;
 		icalcomponent_kind child_kind;
 		icalcomponent *subcomp;
+		icalcomponent *vcal_comp;
 
+		vcal_comp = icalcomp;
 		subcomp = icalcomponent_get_first_component (
-			icalcomp, ICAL_ANY_COMPONENT);
+			vcal_comp, ICAL_ANY_COMPONENT);
 		while (subcomp) {
 			child_kind = icalcomponent_isa (subcomp);
 			if (child_kind == ICAL_VEVENT_COMPONENT ||
@@ -1066,24 +1302,14 @@ selection_received (GtkWidget *invisible,
 					tmp_comp, icalcomponent_new_clone (subcomp));
 				cal_component_set_uid (tmp_comp, uid);
 
+				cal_client_update_object (
+					calendar_model_get_cal_client (cal_table->model),
+					tmp_comp);
 				free (uid);
 				gtk_object_unref (GTK_OBJECT (tmp_comp));
-
-				num_found++;
 			}
 			subcomp = icalcomponent_get_next_component (
-				icalcomp, ICAL_ANY_COMPONENT);
-		}
-
-		if (num_found) {
-			comp = cal_component_new ();
-			cal_component_set_icalcomponent (comp, icalcomp);
-
-			cal_client_update_object (
-				calendar_model_get_cal_client (cal_table->model),
-				comp);
-
-			gtk_object_unref (GTK_OBJECT (comp));
+				vcal_comp, ICAL_ANY_COMPONENT);
 		}
 	}
 	else {
@@ -1098,6 +1324,8 @@ selection_received (GtkWidget *invisible,
 			comp);
 		gtk_object_unref (GTK_OBJECT (comp));
 	}
+
+	calendar_model_set_status_message (e_calendar_table_get_model (cal_table), NULL);
 }
 
 
@@ -1134,7 +1362,7 @@ e_calendar_table_get_current_time (ECellDateEdit *ecde, gpointer data)
 #ifdef TRANSLATORS_ONLY
 
 static char *test[] = {
-    N_("Click here to add a task")
+    N_("Click to add a task")
 };
 
 #endif
