@@ -39,6 +39,7 @@
 #include <ical.h>
 #include <Evolution-Composer.h>
 #include <e-util/e-time-utils.h>
+#include <e-util/e-config-listener.h>
 #include <cal-util/timeutil.h>
 #include <cal-util/cal-util.h>
 #include "calendar-config.h"
@@ -68,7 +69,7 @@ static icalproperty_method itip_methods_enum[] = {
     ICAL_METHOD_DECLINECOUNTER,
 };
 
-static Bonobo_ConfigDatabase db = NULL;
+static EConfigListener *config = NULL;
 
 static ItipAddress *
 get_address (long num) 
@@ -80,11 +81,11 @@ get_address (long num)
 
 	/* get the identity info */
 	path = g_strdup_printf ("/Mail/Accounts/identity_name_%ld", num);
-	a->name = bonobo_config_get_string (db, path, NULL);
+	a->name = e_config_listener_get_string_with_default (config, path, NULL, NULL);
 	g_free (path);
 
 	path = g_strdup_printf ("/Mail/Accounts/identity_address_%ld", num);
-	a->address = bonobo_config_get_string (db, path, NULL);
+	a->address = e_config_listener_get_string_with_default (config, path, NULL, NULL);
 	a->address = g_strstrip (a->address);
 	g_free (path);
 
@@ -96,28 +97,14 @@ get_address (long num)
 GList *
 itip_addresses_get (void)
 {
-
-	CORBA_Environment ev;
 	GList *addresses = NULL;
 	glong len, def, i;
 
-	if (db == NULL) {
-		CORBA_exception_init (&ev);
- 
-		db = bonobo_get_object ("wombat:", 
-					"Bonobo/ConfigDatabase", 
-					&ev);
+	if (config == NULL)
+		config = e_config_listener_new ();
 	
-		if (BONOBO_EX (&ev) || db == CORBA_OBJECT_NIL) {
-			CORBA_exception_free (&ev);
-			return NULL;
-		}
-		
-		CORBA_exception_free (&ev);
-	}
-	
-	len = bonobo_config_get_long_with_default (db, "/Mail/Accounts/num", 0, NULL);
-	def = bonobo_config_get_long_with_default (db, "/Mail/Accounts/default_account", 0, NULL);
+	len = e_config_listener_get_long_with_default (config, "/Mail/Accounts/num", 0, NULL);
+	def = e_config_listener_get_long_with_default (config, "/Mail/Accounts/default_account", 0, NULL);
 
 	for (i = 0; i < len; i++) {
 		ItipAddress *a;
@@ -135,26 +122,14 @@ itip_addresses_get (void)
 ItipAddress *
 itip_addresses_get_default (void)
 {
-	CORBA_Environment ev;
 	ItipAddress *a;
 	glong def;
 
-	if (db == NULL) {
-		CORBA_exception_init (&ev);
- 
-		db = bonobo_get_object ("wombat:", 
-					"Bonobo/ConfigDatabase", 
-					&ev);
+	if (config == NULL)
+		config = e_config_listener_new ();
 	
-		if (BONOBO_EX (&ev) || db == CORBA_OBJECT_NIL) {
-			CORBA_exception_free (&ev);
-			return NULL;
-		}
-		
-		CORBA_exception_free (&ev);
-	}
+	def = e_config_listener_get_long_with_default (config, "/Mail/Accounts/default_account", 0, NULL);
 
-	def = bonobo_config_get_long_with_default (db, "/Mail/Accounts/default_account", 0, NULL);
 	a = get_address (def);
 	a->default_address = TRUE;
 
@@ -183,20 +158,31 @@ itip_addresses_free (GList *addresses)
 }
 
 gboolean
-itip_organizer_is_user (CalComponent *comp)
+itip_organizer_is_user (CalComponent *comp, CalClient *client)
 {
 	CalComponentOrganizer organizer;
 	GList *addresses, *l;
 	const char *strip;
 	gboolean user_org = FALSE;
-	
+
 	if (!cal_component_has_organizer (comp))
 		return FALSE;
 
 	cal_component_get_organizer (comp, &organizer);
 	if (organizer.value != NULL) {
+
 		strip = itip_strip_mailto (organizer.value);
 
+		if (cal_client_get_static_capability (client, "organizer-not-email-address")) { 
+			const char *email;
+			
+			email = cal_client_get_cal_address (client);
+			if (email && !g_strcasecmp (email, strip))
+				return TRUE;
+
+			return FALSE;
+		}
+	
 		addresses = itip_addresses_get ();
 		for (l = addresses; l != NULL; l = l->next) {
 			ItipAddress *a = l->data;
@@ -254,6 +240,21 @@ itip_strip_mailto (const gchar *address)
 	return address;
 }
 
+static char *
+get_label (struct icaltimetype *tt)
+{
+        char buffer[1000];
+        struct tm tmp_tm;
+
+	tmp_tm = icaltimetype_to_tm (tt);
+        e_time_format_date_and_time (&tmp_tm,
+                                     calendar_config_get_24_hour_format (),
+                                     FALSE, FALSE,
+                                     buffer, 1000);
+
+        return g_strdup (buffer);
+}
+
 typedef struct {
 	GHashTable *tzids;
 	icalcomponent *icomp;	
@@ -261,14 +262,127 @@ typedef struct {
 	icalcomponent *zones;
 } ItipUtilTZData;
 
+
+static void
+foreach_tzid_callback (icalparameter *param, gpointer data)
+{
+	ItipUtilTZData *tz_data = data;	
+	const char *tzid;
+	icaltimezone *zone = NULL;
+	icalcomponent *vtimezone_comp;
+
+	/* Get the TZID string from the parameter. */
+	tzid = icalparameter_get_tzid (param);
+	if (!tzid || g_hash_table_lookup (tz_data->tzids, tzid))
+		return;
+
+	/* Look for the timezone */
+	if (tz_data->zones != NULL)
+		zone = icalcomponent_get_timezone (tz_data->zones, tzid);
+	if (zone == NULL)
+		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	if (zone == NULL && tz_data->client != NULL)
+		cal_client_get_timezone (tz_data->client, tzid, &zone);
+	if (zone == NULL)
+		return;
+
+	/* Convert it to a string and add it to the hash. */
+	vtimezone_comp = icaltimezone_get_component (zone);
+	if (!vtimezone_comp)
+		return;
+
+	icalcomponent_add_component (tz_data->icomp, icalcomponent_new_clone (vtimezone_comp));
+	g_hash_table_insert (tz_data->tzids, (char *)tzid, (char *)tzid);	
+}
+
+static icalcomponent *
+comp_toplevel_with_zones (CalComponentItipMethod method, CalComponent *comp, CalClient *client, icalcomponent *zones)
+{
+	icalcomponent *top_level, *icomp;
+	icalproperty *prop;
+	icalvalue *value;
+	ItipUtilTZData tz_data;
+
+	top_level = cal_util_new_top_level ();
+
+	prop = icalproperty_new (ICAL_METHOD_PROPERTY);
+	value = icalvalue_new_method (itip_methods_enum[method]);
+	icalproperty_set_value (prop, value);
+	icalcomponent_add_property (top_level, prop);
+
+	icomp = cal_component_get_icalcomponent (comp);
+	icomp = icalcomponent_new_clone (icomp);
+	
+	tz_data.tzids = g_hash_table_new (g_str_hash, g_str_equal);
+	tz_data.icomp = top_level;
+	tz_data.client = client;
+	tz_data.zones = zones;
+	icalcomponent_foreach_tzid (icomp, foreach_tzid_callback, &tz_data);
+	g_hash_table_destroy (tz_data.tzids);
+
+	icalcomponent_add_component (top_level, icomp);
+
+	return top_level;
+}
+
+static gboolean
+users_has_attendee (GList *users, const char *address)
+{
+	GList *l;
+
+	for (l = users; l != NULL; l = l->next) {
+		if (!g_strcasecmp (address, l->data))
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+static CORBA_char *
+comp_from (CalComponentItipMethod method, CalComponent *comp)
+{
+	CalComponentOrganizer organizer;
+	CalComponentAttendee *attendee;
+	GSList *attendees;
+	CORBA_char *str;
+	
+	switch (method) {
+	case CAL_COMPONENT_METHOD_PUBLISH:
+		return CORBA_string_dup ("");
+		
+	case CAL_COMPONENT_METHOD_REQUEST:
+	case CAL_COMPONENT_METHOD_CANCEL:
+	case CAL_COMPONENT_METHOD_ADD:
+		cal_component_get_organizer (comp, &organizer);
+		if (organizer.value == NULL) {
+			e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
+				  _("An organizer must be set."));
+			return NULL;
+		}
+
+		return CORBA_string_dup (itip_strip_mailto (organizer.value));
+
+	default:
+		if (!cal_component_has_attendees (comp))
+			return CORBA_string_dup ("");
+
+		cal_component_get_attendee_list (comp, &attendees);
+		attendee = attendees->data;
+		str = CORBA_string_dup (attendee->value ? itip_strip_mailto (attendee->value) : "");
+		cal_component_free_attendee_list (attendees);
+
+		return str;
+	}
+}
+
 static GNOME_Evolution_Composer_RecipientList *
-comp_to_list (CalComponentItipMethod method, CalComponent *comp)
+comp_to_list (CalComponentItipMethod method, CalComponent *comp, GList *users)
 {
 	GNOME_Evolution_Composer_RecipientList *to_list;
 	GNOME_Evolution_Composer_Recipient *recipient;
 	CalComponentOrganizer organizer;
 	GSList *attendees, *l;
-	gint cntr, len;
+	gint len;
 
 	switch (method) {
 	case CAL_COMPONENT_METHOD_REQUEST:
@@ -284,18 +398,23 @@ comp_to_list (CalComponentItipMethod method, CalComponent *comp)
 		
 		to_list = GNOME_Evolution_Composer_RecipientList__alloc ();
 		to_list->_maximum = len;
-		to_list->_length = len;
+		to_list->_length = 0;
 		to_list->_buffer = CORBA_sequence_GNOME_Evolution_Composer_Recipient_allocbuf (len);
 		
-		for (cntr = 0, l = attendees; cntr < len; cntr++, l = l->next) {
+		for (l = attendees; l != NULL; l = l->next) {
 			CalComponentAttendee *att = l->data;
-			
-			recipient = &(to_list->_buffer[cntr]);
+
+			if (users_has_attendee (users, att->value))
+				continue;
+
+			recipient = &(to_list->_buffer[to_list->_length]);
 			if (att->cn)
 				recipient->name = CORBA_string_dup (att->cn);
 			else
 				recipient->name = CORBA_string_dup ("");
 			recipient->address = CORBA_string_dup (itip_strip_mailto (att->value));
+			
+			to_list->_length++;
 		}
 		cal_component_free_attendee_list (attendees);
 		break;
@@ -440,99 +559,90 @@ comp_content_type (CalComponent *comp, CalComponentItipMethod method)
 
 }
 
-static void
-foreach_tzid_callback (icalparameter *param, gpointer data)
+static CORBA_char *
+comp_filename (CalComponent *comp)
 {
-	ItipUtilTZData *tz_data = data;	
-	const char *tzid;
-	icaltimezone *zone = NULL;
-	icalcomponent *vtimezone_comp;
-
-	/* Get the TZID string from the parameter. */
-	tzid = icalparameter_get_tzid (param);
-	if (!tzid || g_hash_table_lookup (tz_data->tzids, tzid))
-		return;
-
-	/* Look for the timezone */
-	if (tz_data->zones != NULL)
-		zone = icalcomponent_get_timezone (tz_data->zones, tzid);
-	if (zone == NULL)
-		zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
-	if (zone == NULL && tz_data->client != NULL)
-		cal_client_get_timezone (tz_data->client, tzid, &zone);
-	if (zone == NULL)
-		return;
-
-	/* Convert it to a string and add it to the hash. */
-	vtimezone_comp = icaltimezone_get_component (zone);
-	if (!vtimezone_comp)
-		return;
-
-	icalcomponent_add_component (tz_data->icomp, icalcomponent_new_clone (vtimezone_comp));
-	g_hash_table_insert (tz_data->tzids, (char *)tzid, (char *)tzid);	
+        switch (cal_component_get_vtype (comp)) {
+        case CAL_COMPONENT_FREEBUSY:
+                return CORBA_string_dup ("freebusy.ifb");
+        default:
+                return CORBA_string_dup ("calendar.ics");
+        }
 }
 
-static char *
-comp_string (CalComponentItipMethod method, CalComponent *comp, CalClient *client, icalcomponent *zones)
+static CORBA_char *
+comp_description (CalComponent *comp)
 {
-	icalcomponent *top_level, *icomp;
-	icalproperty *prop;
-	icalvalue *value;
-	gchar *ical_string;
-	ItipUtilTZData tz_data;
+        CORBA_char *description;
+        CalComponentDateTime dt;
+        char *start = NULL, *end = NULL;
+
+        switch (cal_component_get_vtype (comp)) {
+        case CAL_COMPONENT_EVENT:
+                return CORBA_string_dup (U_("Event information"));
+        case CAL_COMPONENT_TODO:
+                return CORBA_string_dup (U_("Task information"));
+        case CAL_COMPONENT_JOURNAL:
+                return CORBA_string_dup (U_("Journal information"));
+        case CAL_COMPONENT_FREEBUSY:
+                cal_component_get_dtstart (comp, &dt);
+                if (dt.value)
+                        start = get_label (dt.value);
+		cal_component_free_datetime (&dt);
+
+		cal_component_get_dtend (comp, &dt);
+		if (dt.value)
+			end = get_label (dt.value);
+		cal_component_free_datetime (&dt);
+
+                if (start != NULL && end != NULL) {
+                        char *tmp, *tmp_utf;
+                        tmp = g_strdup_printf (_("Free/Busy information (%s to %s)"), start, end);
+                        tmp_utf = e_utf8_from_locale_string (tmp);
+                        description = CORBA_string_dup (tmp_utf);
+                        g_free (tmp_utf);
+                        g_free (tmp);
+                } else {
+                        description = CORBA_string_dup (U_("Free/Busy information"));
+                }
+                g_free (start);
+                g_free (end);
+                return description;
+        default:
+                return CORBA_string_dup (U_("iCalendar information"));
+        }
+}
+
+static gboolean
+comp_server_send (CalComponentItipMethod method, CalComponent *comp, CalClient *client, 
+		  icalcomponent *zones, GList **users)
+{
+	CalClientSendResult result;
+	icalcomponent *top_level, *new_top_level = NULL;
+	char error_msg[256];
+	gboolean retval = FALSE;
+	
+	top_level = comp_toplevel_with_zones (method, comp, client, zones);
+	result = cal_client_send_object (client, top_level, &new_top_level, users, error_msg);
+
+	if (result == CAL_CLIENT_SEND_SUCCESS) {
+		icalcomponent *ical_comp;
 		
-	top_level = cal_util_new_top_level ();
+		ical_comp = icalcomponent_get_inner (new_top_level);
+		icalcomponent_remove_component (new_top_level, ical_comp);
+		cal_component_set_icalcomponent (comp, ical_comp);
+		icalcomponent_free (new_top_level);
+		
+		retval = TRUE;
+	} else if (result == CAL_CLIENT_SEND_BUSY) {
+		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR, error_msg);
 
-	prop = icalproperty_new (ICAL_METHOD_PROPERTY);
-	value = icalvalue_new_method (itip_methods_enum[method]);
-	icalproperty_set_value (prop, value);
-	icalcomponent_add_property (top_level, prop);
-
-	icomp = cal_component_get_icalcomponent (comp);
-
-	if (method == CAL_COMPONENT_METHOD_REPLY) {
-		struct icaltimetype dtstamp;
-		gboolean add_it = FALSE;
-
-		/* workaround for Outlook expecting a X-MICROSOFT-CDO-REPLYTIME
-		   on every METHOD=REPLY message. If the component has any of
-		   the X-MICROSOFT-* properties, we add the REPLYTIME one */
-		prop = icalcomponent_get_first_property (icomp, ICAL_X_PROPERTY);
-		while (prop) {
-			const char *x_name;
-
-			x_name = icalproperty_get_x_name (prop);
-			if (!strncmp (x_name, "X-MICROSOFT-", strlen ("X-MICROSOFT-"))) {
-				add_it = TRUE;
-				break;
-			}
-			prop = icalcomponent_get_next_property (icomp, ICAL_X_PROPERTY);
-		}
-
-		if (add_it) {
-			dtstamp = icaltime_from_timet_with_zone (
-				time (NULL), 0, icaltimezone_get_utc_timezone ());
-			prop = icalproperty_new_x (icaltime_as_ical_string (dtstamp));
-			icalproperty_set_x_name (prop, "X-MICROSOFT-CDO-REPLYTIME");
-			icalcomponent_add_property (icomp, prop);
-		}
+		retval = FALSE;
 	}
-		
-	/* Add the timezones */
-	tz_data.tzids = g_hash_table_new (g_str_hash, g_str_equal);
-	tz_data.icomp = top_level;
-	tz_data.client = client;
-	tz_data.zones = zones;
-	icalcomponent_foreach_tzid (icomp, foreach_tzid_callback, &tz_data);
-	g_hash_table_destroy (tz_data.tzids);
 
-	icalcomponent_add_component (top_level, icomp);
-	ical_string = icalcomponent_as_ical_string (top_level);
-	icalcomponent_remove_component (top_level, icomp);
-	
 	icalcomponent_free (top_level);
-	
-	return ical_string;	
+
+	return retval;
 }
 
 static gboolean
@@ -597,7 +707,7 @@ comp_limit_attendees (CalComponent *comp)
 }
 
 static void
-comp_sentby (CalComponent *comp)
+comp_sentby (CalComponent *comp, CalClient *client)
 {
 	CalComponentOrganizer organizer;
 	
@@ -617,7 +727,7 @@ comp_sentby (CalComponent *comp)
 		return;
 	}
 
-	if (!itip_organizer_is_user (comp) && !itip_sentby_is_user (comp)) {
+	if (!itip_organizer_is_user (comp, client) && !itip_sentby_is_user (comp)) {
 		ItipAddress *a = itip_addresses_get_default ();
 		
 		organizer.value = g_strdup (organizer.value);
@@ -712,7 +822,7 @@ comp_minimal (CalComponent *comp, gboolean attendee)
 }
 
 static CalComponent *
-comp_compliant (CalComponentItipMethod method, CalComponent *comp)
+comp_compliant (CalComponentItipMethod method, CalComponent *comp, CalClient *client, icalcomponent *zones)
 {
 	CalComponent *clone, *temp_clone;
 	struct icaltimetype itt;
@@ -722,6 +832,51 @@ comp_compliant (CalComponentItipMethod method, CalComponent *comp)
 					     icaltimezone_get_utc_timezone ());
 	cal_component_set_dtstamp (clone, &itt);
 
+	/* Make UNTIL date a datetime in a simple recurrence */
+	if (cal_component_has_recurrences (clone)
+	    && cal_component_has_simple_recurrence (clone)) {
+		GSList *rrule_list;
+		struct icalrecurrencetype *r;
+		
+		cal_component_get_rrule_list (clone, &rrule_list);
+		r = rrule_list->data;
+
+		if (!icaltime_is_null_time (r->until) && r->until.is_date) {
+			CalComponentDateTime dt;
+			icaltimezone *from_zone = NULL, *to_zone;
+			
+			cal_component_get_dtstart (clone, &dt);
+
+			if (dt.value->is_date) {
+				from_zone = icaltimezone_get_builtin_timezone (calendar_config_get_timezone ());
+			} else if (dt.tzid == NULL) {
+				from_zone = icaltimezone_get_utc_timezone ();
+			} else {
+				if (zones != NULL)
+					from_zone = icalcomponent_get_timezone (zones, dt.tzid);
+				if (from_zone == NULL)
+					from_zone = icaltimezone_get_builtin_timezone_from_tzid (dt.tzid);
+				if (from_zone == NULL && client != NULL)
+					cal_client_get_timezone (client, dt.tzid, &from_zone);
+			}
+			
+			to_zone = icaltimezone_get_utc_timezone ();
+
+			r->until.hour = dt.value->hour;
+			r->until.minute = dt.value->minute;
+			r->until.second = dt.value->second;
+			r->until.is_date = FALSE;
+			
+			icaltimezone_convert_time (&r->until, from_zone, to_zone);
+			r->until.is_utc = TRUE;
+
+			cal_component_set_rrule_list (clone, rrule_list);
+			cal_component_abort_sequence (clone);
+		}
+
+		cal_component_free_recur_list (rrule_list);
+	}
+	
 	/* We delete incoming alarms anyhow, and this helps with outlook */
 	cal_component_remove_all_alarms (clone);
 
@@ -731,17 +886,16 @@ comp_compliant (CalComponentItipMethod method, CalComponent *comp)
 	/* Comply with itip spec */
 	switch (method) {
 	case CAL_COMPONENT_METHOD_PUBLISH:
-		comp_sentby (clone);
+		comp_sentby (clone, client);
 		cal_component_set_attendee_list (clone, NULL);
 		break;
 	case CAL_COMPONENT_METHOD_REQUEST:
-		comp_sentby (clone);
+		comp_sentby (clone, client);
 		break;
 	case CAL_COMPONENT_METHOD_CANCEL:
-		comp_sentby (clone);
+		comp_sentby (clone, client);
 		break;	
 	case CAL_COMPONENT_METHOD_REPLY:
-		comp_minimal (clone, TRUE);
 		break;
 	case CAL_COMPONENT_METHOD_ADD:
 		break;
@@ -760,41 +914,58 @@ comp_compliant (CalComponentItipMethod method, CalComponent *comp)
 		clone = temp_clone;
 		break;
 	default:
+		break;
 	}
 
 	return clone;
 }
 
-void
+gboolean
 itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		CalClient *client, icalcomponent *zones)
 {
 	BonoboObjectClient *bonobo_server;
 	GNOME_Evolution_Composer composer_server;
 	CalComponent *comp = NULL;
+	icalcomponent *top_level = NULL;
+	GList *users;
 	GNOME_Evolution_Composer_RecipientList *to_list = NULL;
 	GNOME_Evolution_Composer_RecipientList *cc_list = NULL;
 	GNOME_Evolution_Composer_RecipientList *bcc_list = NULL;
 	CORBA_char *subject = NULL, *body = NULL, *content_type = NULL;
-	CORBA_char *filename = NULL, *description = NULL;
+	CORBA_char *from = NULL, *filename = NULL, *description = NULL;
 	GNOME_Evolution_Composer_AttachmentData *attach_data = NULL;
 	char *ical_string;
 	CORBA_Environment ev;
+	gboolean retval = FALSE;
 	
 	CORBA_exception_init (&ev);
 
 	/* Obtain an object reference for the Composer. */
 	bonobo_server = bonobo_object_activate (GNOME_EVOLUTION_COMPOSER_OAFIID, 0);
-	g_return_if_fail (bonobo_server != NULL);
+	g_return_val_if_fail (bonobo_server != NULL, FALSE);
 	composer_server = BONOBO_OBJREF (bonobo_server);
-
-	comp = comp_compliant (method, send_comp);
+	
+	/* Give the server a chance to manipulate the comp */
+	if (method != CAL_COMPONENT_METHOD_PUBLISH) {
+		if (!comp_server_send (method, send_comp, client, zones, &users))
+			goto cleanup;
+	}
+	
+	/* Tidy up the comp */
+	comp = comp_compliant (method, send_comp, client, zones);
 	if (comp == NULL)
 		goto cleanup;
-	
-	to_list = comp_to_list (method, comp);
-	if (to_list == NULL)
-		goto cleanup;
+
+	/* Recipients */
+	to_list = comp_to_list (method, comp, users);
+	if (method != CAL_COMPONENT_METHOD_PUBLISH) {
+		if (to_list == NULL || to_list->_length == 0) {
+			/* We sent them all via the server */
+			retval = TRUE;
+			goto cleanup;
+		}
+	}
 	
 	cc_list = GNOME_Evolution_Composer_RecipientList__alloc ();
 	cc_list->_maximum = cc_list->_length = 0;
@@ -804,27 +975,55 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 	/* Subject information */
 	subject = comp_subject (method, comp);
 	
+	/* From address */
+	from = comp_from (method, comp);
+
 	/* Set recipients, subject */
-	GNOME_Evolution_Composer_setHeaders (composer_server, to_list, cc_list, bcc_list, subject, &ev);
+	GNOME_Evolution_Composer_setHeaders (composer_server, from, to_list, cc_list, bcc_list, subject, &ev);
 	if (BONOBO_EX (&ev)) {		
 		g_warning ("Unable to set composer headers while sending iTip message");
 		goto cleanup;
 	}
 
+
 	/* Content type */
 	content_type = comp_content_type (comp, method);
 
-	ical_string = comp_string (method, comp, client, zones);
-	attach_data = GNOME_Evolution_Composer_AttachmentData__alloc ();
-	attach_data->_length = strlen (ical_string) + 1;
-	attach_data->_maximum = attach_data->_length;	
-	attach_data->_buffer = CORBA_sequence_CORBA_char_allocbuf (attach_data->_length);
-	strcpy (attach_data->_buffer, ical_string);
+	top_level = comp_toplevel_with_zones (method, comp, client, zones);
+	ical_string = icalcomponent_as_ical_string (top_level);
 
-	GNOME_Evolution_Composer_setBody (composer_server, ical_string, content_type, &ev);
+	if (cal_component_get_vtype (comp) == CAL_COMPONENT_EVENT) {
+		GNOME_Evolution_Composer_setBody (composer_server, ical_string, content_type, &ev);
+	} else {
+		GNOME_Evolution_Composer_setMultipartType (composer_server, GNOME_Evolution_Composer_MIXED, &ev);
+		if (BONOBO_EX (&ev)) {
+			g_warning ("Unable to set multipart type while sending iTip message");
+			goto cleanup;
+		}
 
+		filename = comp_filename (comp);
+		description = comp_description (comp);
+
+		GNOME_Evolution_Composer_setBody (composer_server, description, "text/plain", &ev);
+		if (BONOBO_EX (&ev)) {
+			g_warning ("Unable to set body text while sending iTip message");
+			goto cleanup;
+		}
+
+		attach_data = GNOME_Evolution_Composer_AttachmentData__alloc ();
+		attach_data->_length = strlen (ical_string) + 1;
+		attach_data->_maximum = attach_data->_length;	
+		attach_data->_buffer = CORBA_sequence_CORBA_char_allocbuf (attach_data->_length);
+		strcpy (attach_data->_buffer, ical_string);
+
+		GNOME_Evolution_Composer_attachData (composer_server,
+						     content_type, filename, description,
+						     TRUE, attach_data,
+						     &ev);
+	}
+		
 	if (BONOBO_EX (&ev)) {
-		g_warning ("Unable to attach data to the composer while sending iTip message");
+		g_warning ("Unable to place iTip message in composer");
 		goto cleanup;
 	}
 	
@@ -832,10 +1031,14 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		GNOME_Evolution_Composer_show (composer_server, &ev);
 		if (BONOBO_EX (&ev))
 			g_warning ("Unable to show the composer while sending iTip message");
+		else
+			retval = TRUE;
 	} else {		
 		GNOME_Evolution_Composer_send (composer_server, &ev);
 		if (BONOBO_EX (&ev))
 			g_warning ("Unable to send iTip message");
+		else
+			retval = TRUE;
 	}
 	
  cleanup:
@@ -843,7 +1046,9 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 
 	if (comp != NULL)
 		gtk_object_unref (GTK_OBJECT (comp));
-		
+	if (top_level != NULL)
+		icalcomponent_free (top_level);
+
 	if (to_list != NULL)
 		CORBA_free (to_list);
 	if (cc_list != NULL)
@@ -851,6 +1056,8 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 	if (bcc_list != NULL)
 		CORBA_free (bcc_list);
 
+	if (from != NULL)
+		CORBA_free (from);
 	if (subject != NULL)
 		CORBA_free (subject);
 	if (body != NULL)
@@ -861,7 +1068,11 @@ itip_send_comp (CalComponentItipMethod method, CalComponent *send_comp,
 		CORBA_free (filename);
 	if (description != NULL)
 		CORBA_free (description);
-	if (attach_data != NULL)
+	if (attach_data != NULL) {
+		CORBA_free (attach_data->_buffer);
 		CORBA_free (attach_data);
+	}
+
+	return retval;
 }
 
