@@ -92,7 +92,10 @@ static void cal_backend_file_destroy (GtkObject *object);
 
 static const char *cal_backend_file_get_uri (CalBackend *backend);
 static gboolean cal_backend_file_is_read_only (CalBackend *backend);
-static const char *cal_backend_file_get_email_address (CalBackend *backend);
+static const char *cal_backend_file_get_cal_address (CalBackend *backend);
+static const char *cal_backend_file_get_alarm_email_address (CalBackend *backend);
+static const char *cal_backend_file_get_ldap_attribute (CalBackend *backend);
+static const char *cal_backend_file_get_static_capabilities (CalBackend *backend);
 static CalBackendOpenStatus cal_backend_file_open (CalBackend *backend,
 						   const char *uristr,
 						   gboolean only_if_exists);
@@ -105,6 +108,7 @@ static CalMode cal_backend_file_get_mode (CalBackend *backend);
 static void cal_backend_file_set_mode (CalBackend *backend, CalMode mode);
 
 static int cal_backend_file_get_n_objects (CalBackend *backend, CalObjType type);
+static char *cal_backend_file_get_default_object (CalBackend *backend, CalObjType type);
 static char *cal_backend_file_get_object (CalBackend *backend, const char *uid);
 static CalComponent *cal_backend_file_get_object_component (CalBackend *backend, const char *uid);
 static char *cal_backend_file_get_timezone_object (CalBackend *backend, const char *tzid);
@@ -123,8 +127,9 @@ static GNOME_Evolution_Calendar_CalComponentAlarms *cal_backend_file_get_alarms_
 	time_t start, time_t end, gboolean *object_found);
 
 static CalBackendResult cal_backend_file_update_objects (CalBackend *backend,
-							 const char *calobj);
-static CalBackendResult cal_backend_file_remove_object (CalBackend *backend, const char *uid);
+							 const char *calobj,
+							 CalObjModType mod);
+static CalBackendResult cal_backend_file_remove_object (CalBackend *backend, const char *uid, CalObjModType mod);
 
 static CalBackendSendResult cal_backend_file_send_object (CalBackend *backend, 
 							  const char *calobj, gchar **new_calobj,
@@ -137,6 +142,7 @@ static gboolean cal_backend_file_set_default_timezone (CalBackend *backend,
 						       const char *tzid);
 
 static void notify_categories_changed (CalBackendFile *cbfile);
+static void notify_error (CalBackendFile *cbfile, const char *message);
 
 static CalBackendClass *parent_class;
 
@@ -190,13 +196,17 @@ cal_backend_file_class_init (CalBackendFileClass *class)
 
 	backend_class->get_uri = cal_backend_file_get_uri;
 	backend_class->is_read_only = cal_backend_file_is_read_only;
-	backend_class->get_email_address = cal_backend_file_get_email_address;
+	backend_class->get_cal_address = cal_backend_file_get_cal_address;
+	backend_class->get_alarm_email_address = cal_backend_file_get_alarm_email_address;
+	backend_class->get_ldap_attribute = cal_backend_file_get_ldap_attribute;
+	backend_class->get_static_capabilities = cal_backend_file_get_static_capabilities;
 	backend_class->open = cal_backend_file_open;
 	backend_class->is_loaded = cal_backend_file_is_loaded;
 	backend_class->get_query = cal_backend_file_get_query;
 	backend_class->get_mode = cal_backend_file_get_mode;
 	backend_class->set_mode = cal_backend_file_set_mode;	
 	backend_class->get_n_objects = cal_backend_file_get_n_objects;
+	backend_class->get_default_object = cal_backend_file_get_default_object;
 	backend_class->get_object = cal_backend_file_get_object;
 	backend_class->get_object_component = cal_backend_file_get_object_component;
 	backend_class->get_timezone_object = cal_backend_file_get_timezone_object;
@@ -284,7 +294,7 @@ save (CalBackendFile *cbfile)
 	CalBackendFilePrivate *priv;
 	GnomeVFSURI *uri, *backup_uri;
 	GnomeVFSHandle *handle = NULL;
-	GnomeVFSResult result;
+	GnomeVFSResult result = GNOME_VFS_ERROR_BAD_FILE;
 	GnomeVFSFileSize out;
 	gchar *tmp, *backup_uristr;
 	char *buf;
@@ -474,7 +484,7 @@ cal_backend_file_is_read_only (CalBackend *backend)
 
 /* Get_email_address handler for the file backend */
 static const char *
-cal_backend_file_get_email_address (CalBackend *backend)
+cal_backend_file_get_cal_address (CalBackend *backend)
 {
 	/* A file backend has no particular email address associated
 	 * with it (although that would be a useful feature some day).
@@ -482,6 +492,27 @@ cal_backend_file_get_email_address (CalBackend *backend)
 	return NULL;
 }
 
+static const char *
+cal_backend_file_get_alarm_email_address (CalBackend *backend)
+{
+	/* A file backend has no particular email address associated
+	 * with it (although that would be a useful feature some day).
+	 */
+	return NULL;
+}
+
+static const char *
+cal_backend_file_get_ldap_attribute (CalBackend *backend)
+{
+	return NULL;
+}
+
+static const char *
+cal_backend_file_get_static_capabilities (CalBackend *backend)
+{
+	return "no-email-alarms";
+}
+
 /* Used from g_hash_table_foreach(), adds a category name to the sequence */
 static void
 add_category_cb (gpointer key, gpointer value, gpointer data)
@@ -1042,6 +1073,41 @@ cal_backend_file_get_n_objects (CalBackend *backend, CalObjType type)
 	return n;
 }
 
+/* Get_object handler for the file backend */
+static char *
+cal_backend_file_get_default_object (CalBackend *backend, CalObjType type)
+{
+	CalBackendFile *cbfile;
+	CalBackendFilePrivate *priv;
+	CalComponent *comp;
+	char *calobj;
+	
+	cbfile = CAL_BACKEND_FILE (backend);
+	priv = cbfile->priv;
+
+	comp = cal_component_new ();
+	
+	switch (type) {
+	case CALOBJ_TYPE_EVENT:
+		cal_component_set_new_vtype (comp, CAL_COMPONENT_EVENT);
+		break;
+	case CALOBJ_TYPE_TODO:
+		cal_component_set_new_vtype (comp, CAL_COMPONENT_TODO);
+		break;
+	case CALOBJ_TYPE_JOURNAL:
+		cal_component_set_new_vtype (comp, CAL_COMPONENT_JOURNAL);
+		break;
+	default:
+		gtk_object_unref (GTK_OBJECT (comp));
+		return NULL;
+	}
+	
+	calobj = cal_component_get_as_string (comp);
+	gtk_object_unref (GTK_OBJECT (comp));
+
+	return calobj;
+}
+
 /* Get_object handler for the file backend */
 static char *
 cal_backend_file_get_object (CalBackend *backend, const char *uid)
@@ -1589,6 +1655,8 @@ cal_backend_file_get_alarms_in_range (CalBackend *backend,
 	GSList *comp_alarms;
 	GSList *l;
 	int i;
+	CalAlarmAction omit[] = {-1};
+	
 	GNOME_Evolution_Calendar_CalComponentAlarmsSeq *seq;
 
 	cbfile = CAL_BACKEND_FILE (backend);
@@ -1604,11 +1672,11 @@ cal_backend_file_get_alarms_in_range (CalBackend *backend,
 	n_comp_alarms = 0;
 	comp_alarms = NULL;
 
-	n_comp_alarms += cal_util_generate_alarms_for_list (priv->events, start, end,
+	n_comp_alarms += cal_util_generate_alarms_for_list (priv->events, start, end, omit,
 							    &comp_alarms, resolve_tzid,
 							    priv->icalcomp,
 							    priv->default_zone);
-	n_comp_alarms += cal_util_generate_alarms_for_list (priv->todos, start, end,
+	n_comp_alarms += cal_util_generate_alarms_for_list (priv->todos, start, end, omit,
 							    &comp_alarms, resolve_tzid,
 							    priv->icalcomp,
 							    priv->default_zone);
@@ -1651,6 +1719,7 @@ cal_backend_file_get_alarms_for_object (CalBackend *backend, const char *uid,
 	char *comp_str;
 	GNOME_Evolution_Calendar_CalComponentAlarms *corba_alarms;
 	CalComponentAlarms *alarms;
+	CalAlarmAction omit[] = {-1};
 
 	cbfile = CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -1676,7 +1745,7 @@ cal_backend_file_get_alarms_for_object (CalBackend *backend, const char *uid,
 	corba_alarms->calobj = CORBA_string_dup (comp_str);
 	g_free (comp_str);
 
-	alarms = cal_util_generate_alarms_for_comp (comp, start, end, resolve_tzid, priv->icalcomp, priv->default_zone);
+	alarms = cal_util_generate_alarms_for_comp (comp, start, end, omit, resolve_tzid, priv->icalcomp, priv->default_zone);
 	if (alarms) {
 		cal_backend_util_fill_alarm_instances_seq (&corba_alarms->alarms, alarms->alarms);
 		cal_component_alarms_free (alarms);
@@ -1812,20 +1881,45 @@ cal_backend_file_update_object (CalBackendFile *cbfile,
 	return comp_uid;
 }
 
+static const char*
+cal_backend_file_cancel_object (CalBackendFile *cbfile,
+				icalcomponent *icalcomp)
+{
+	CalComponent *old_comp;
+	icalproperty *uid;
+	const char *comp_uid;
 
+	/* Get the UID, and check it isn't empty. */
+	uid = icalcomponent_get_first_property (icalcomp, ICAL_UID_PROPERTY);
+	if (!uid)
+		return NULL;
+	comp_uid = icalproperty_get_uid (uid);
+	if (!comp_uid || !comp_uid[0])
+		return NULL;
+
+	/* Find the old version of the component. */
+	old_comp = lookup_component (cbfile, comp_uid);
+	if (!old_comp)
+		return NULL;
+
+	/* And remove it */
+	remove_component (cbfile, old_comp);
+	return comp_uid;
+}
 
 /* Update_objects handler for the file backend. */
 static CalBackendResult
-cal_backend_file_update_objects (CalBackend *backend, const char *calobj)
+cal_backend_file_update_objects (CalBackend *backend, const char *calobj, CalObjModType mod)
 {
 	CalBackendFile *cbfile;
 	CalBackendFilePrivate *priv;
 	icalcomponent *toplevel_comp, *icalcomp = NULL;
 	icalcomponent_kind kind;
+	icalproperty_method method;
 	int old_n_categories, new_n_categories;
 	icalcomponent *subcomp;
 	CalBackendResult retval = CAL_BACKEND_RESULT_SUCCESS;
-	GList *comp_uid_list = NULL, *elem;
+	GList *updated_uids = NULL, *removed_uids = NULL, *elem;
 
 	cbfile = CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -1857,6 +1951,8 @@ cal_backend_file_update_objects (CalBackend *backend, const char *calobj)
 		return CAL_BACKEND_RESULT_INVALID_OBJECT;
 	}
 
+	method = icalcomponent_get_method (toplevel_comp);
+
 	/* The list of removed categories must be empty because we are about to
 	 * start a new scanning process.
 	 */
@@ -1877,18 +1973,20 @@ cal_backend_file_update_objects (CalBackend *backend, const char *calobj)
 		    || child_kind == ICAL_VJOURNAL_COMPONENT) {
 			const char *comp_uid;
 
-			comp_uid = cal_backend_file_update_object (cbfile,
-								   subcomp);
-			if (comp_uid) {
-				/* We add a copy of the UID to a list, so we
-				   can emit notification signals later. We do
-				   a g_strdup() in case any of the components
-				   get removed while we are emitting
-				   notification signals. */
-				comp_uid_list = g_list_prepend (comp_uid_list,
-								g_strdup (comp_uid));
+			if (method == ICAL_METHOD_CANCEL) {
+				comp_uid = cal_backend_file_cancel_object (cbfile, subcomp);
+				if (comp_uid) {
+					removed_uids = g_list_prepend (removed_uids,
+								       g_strdup (comp_uid));
+				} else
+					retval = CAL_BACKEND_RESULT_NOT_FOUND;
 			} else {
-				retval = CAL_BACKEND_RESULT_INVALID_OBJECT;
+				comp_uid = cal_backend_file_update_object (cbfile, subcomp);
+				if (comp_uid) {
+					updated_uids = g_list_prepend (updated_uids,
+								       g_strdup (comp_uid));
+				} else
+					retval = CAL_BACKEND_RESULT_INVALID_OBJECT;
 			}
 		}
 		subcomp = icalcomponent_get_next_component (toplevel_comp,
@@ -1906,12 +2004,19 @@ cal_backend_file_update_objects (CalBackend *backend, const char *calobj)
 	/* Now emit notification signals for all of the added components.
 	   We do this after adding them all to make sure the calendar is in a
 	   stable state before emitting signals. */
-	for (elem = comp_uid_list; elem; elem = elem->next) {
+	for (elem = updated_uids; elem; elem = elem->next) {
 		char *comp_uid = elem->data;
 		notify_update (cbfile, comp_uid);
 		g_free (comp_uid);
 	}
-	g_list_free (comp_uid_list);
+	g_list_free (updated_uids);
+
+	for (elem = removed_uids; elem; elem = elem->next) {
+		char *comp_uid = elem->data;
+		notify_remove (cbfile, comp_uid);
+		g_free (comp_uid);
+	}
+	g_list_free (removed_uids);
 
 	if (old_n_categories != new_n_categories ||
 	    g_hash_table_size (priv->removed_categories) != 0) {
@@ -1925,7 +2030,7 @@ cal_backend_file_update_objects (CalBackend *backend, const char *calobj)
 
 /* Remove_object handler for the file backend */
 static CalBackendResult
-cal_backend_file_remove_object (CalBackend *backend, const char *uid)
+cal_backend_file_remove_object (CalBackend *backend, const char *uid, CalObjModType mod)
 {
 	CalBackendFile *cbfile;
 	CalBackendFilePrivate *priv;
