@@ -1,27 +1,28 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* camel-disco-store.c: abstract class for a disconnectable remote store */
+/* camel-disco-store.c: abstract class for a disconnectable store */
 
 /*
  *  Authors: Dan Winship <danw@ximian.com>
  *
  *  Copyright 2001 Ximian, Inc.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -33,12 +34,13 @@
 
 #define CDS_CLASS(o) (CAMEL_DISCO_STORE_CLASS (CAMEL_OBJECT_GET_CLASS (o)))
 
-static CamelRemoteStoreClass *remote_store_class = NULL;
+static CamelStoreClass *parent_class = NULL;
 
 static void disco_construct (CamelService *service, CamelSession *session,
 			     CamelProvider *provider, CamelURL *url,
 			     CamelException *ex);
 static gboolean disco_connect (CamelService *service, CamelException *ex);
+static void disco_cancel_connect (CamelService *service);
 static gboolean disco_disconnect (CamelService *service, gboolean clean, CamelException *ex);
 static CamelFolder *disco_get_folder (CamelStore *store, const char *name,
 				      guint32 flags, CamelException *ex);
@@ -50,25 +52,34 @@ static void set_status (CamelDiscoStore *disco_store,
 			CamelException *ex);
 static gboolean can_work_offline (CamelDiscoStore *disco_store);
 
+static int disco_setv (CamelObject *object, CamelException *ex, CamelArgV *args);
+static int disco_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args);
+
 static void
 camel_disco_store_class_init (CamelDiscoStoreClass *camel_disco_store_class)
 {
+	CamelObjectClass *camel_object_class =
+		CAMEL_OBJECT_CLASS (camel_disco_store_class);
 	CamelServiceClass *camel_service_class =
 		CAMEL_SERVICE_CLASS (camel_disco_store_class);
 	CamelStoreClass *camel_store_class =
 		CAMEL_STORE_CLASS (camel_disco_store_class);
-
-	remote_store_class = CAMEL_REMOTE_STORE_CLASS (camel_type_get_global_classfuncs (camel_remote_store_get_type ()));
-
+	
+	parent_class = CAMEL_STORE_CLASS (camel_type_get_global_classfuncs (camel_store_get_type ()));
+	
 	/* virtual method definition */
 	camel_disco_store_class->set_status = set_status;
 	camel_disco_store_class->can_work_offline = can_work_offline;
-
+	
 	/* virtual method overload */
+	camel_object_class->setv = disco_setv;
+	camel_object_class->getv = disco_getv;
+	
 	camel_service_class->construct = disco_construct;
 	camel_service_class->connect = disco_connect;
 	camel_service_class->disconnect = disco_disconnect;
-
+	camel_service_class->cancel_connect = disco_cancel_connect;
+	
 	camel_store_class->get_folder = disco_get_folder;
 	camel_store_class->get_folder_info = disco_get_folder_info;
 }
@@ -77,10 +88,11 @@ CamelType
 camel_disco_store_get_type (void)
 {
 	static CamelType camel_disco_store_type = CAMEL_INVALID_TYPE;
-
+	
 	if (camel_disco_store_type == CAMEL_INVALID_TYPE) {
 		camel_disco_store_type = camel_type_register (
-			CAMEL_REMOTE_STORE_TYPE, "CamelDiscoStore",
+			CAMEL_STORE_TYPE,
+			"CamelDiscoStore",
 			sizeof (CamelDiscoStore),
 			sizeof (CamelDiscoStoreClass),
 			(CamelObjectClassInitFunc) camel_disco_store_class_init,
@@ -88,10 +100,24 @@ camel_disco_store_get_type (void)
 			NULL,
 			NULL);
 	}
-
+	
 	return camel_disco_store_type;
 }
 
+static int
+disco_setv (CamelObject *object, CamelException *ex, CamelArgV *args)
+{
+	/* CamelDiscoStore doesn't currently have anything to set */
+	return CAMEL_OBJECT_CLASS (parent_class)->setv (object, ex, args);
+}
+
+static int
+disco_getv (CamelObject *object, CamelException *ex, CamelArgGetV *args)
+{
+	/* CamelDiscoStore doesn't currently have anything to get */
+	return CAMEL_OBJECT_CLASS (parent_class)->getv (object, ex, args);
+}
+
 static void
 disco_construct (CamelService *service, CamelSession *session,
 		 CamelProvider *provider, CamelURL *url,
@@ -99,7 +125,7 @@ disco_construct (CamelService *service, CamelSession *session,
 {
 	CamelDiscoStore *disco = CAMEL_DISCO_STORE (service);
 
-	CAMEL_SERVICE_CLASS (remote_store_class)->construct (service, session, provider, url, ex);
+	CAMEL_SERVICE_CLASS (parent_class)->construct (service, session, provider, url, ex);
 	if (camel_exception_is_set (ex))
 		return;
 
@@ -111,11 +137,19 @@ static gboolean
 disco_connect (CamelService *service, CamelException *ex)
 {
 	CamelDiscoStore *store = CAMEL_DISCO_STORE (service);
+	CamelDiscoStoreStatus status;
 
-	if (!CAMEL_SERVICE_CLASS (remote_store_class)->connect (service, ex))
-		return FALSE;
+	status = camel_disco_store_status (store);
+	if (status != CAMEL_DISCO_STORE_OFFLINE) {
+		if (!CAMEL_SERVICE_CLASS (parent_class)->connect (service, ex)) {
+			status = camel_disco_store_status (store);
+			if (status != CAMEL_DISCO_STORE_OFFLINE)
+				return FALSE;
+			camel_exception_clear (ex);
+		}
+	}
 
-	switch (camel_disco_store_status (store)) {
+	switch (status) {
 	case CAMEL_DISCO_STORE_ONLINE:
 	case CAMEL_DISCO_STORE_RESYNCING:
 		if (!CDS_CLASS (service)->connect_online (service, ex))
@@ -142,6 +176,17 @@ disco_connect (CamelService *service, CamelException *ex)
 	return FALSE;
 }
 
+static void
+disco_cancel_connect (CamelService *service)
+{
+	CamelDiscoStore *store = CAMEL_DISCO_STORE (service);
+
+	/* Fall back */
+	store->status = CAMEL_DISCO_STORE_OFFLINE;
+
+	CAMEL_SERVICE_CLASS (parent_class)->cancel_connect (service);
+}
+
 static gboolean
 disco_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
@@ -161,7 +206,7 @@ disco_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 
 	}
 
-	return CAMEL_SERVICE_CLASS (remote_store_class)->disconnect (service, clean, ex);
+	return CAMEL_SERVICE_CLASS (parent_class)->disconnect (service, clean, ex);
 }
 
 static CamelFolder *
@@ -190,7 +235,17 @@ disco_get_folder_info (CamelStore *store, const char *top,
 		       guint32 flags, CamelException *ex)
 {
 	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (store);
-	
+
+	/* Do this first so if we get forced offline, we'll switch to
+	 * the correct branch below. (FIXME: This only works because
+	 * we know that get_folder_info is the first call that the
+	 * mailer makes on a store.)
+	 */
+	if (CAMEL_SERVICE (store)->status == CAMEL_SERVICE_DISCONNECTED) {
+		if (!camel_service_connect (CAMEL_SERVICE (store), ex))
+			return NULL;
+	}
+
 	switch (camel_disco_store_status (disco_store)) {
 	case CAMEL_DISCO_STORE_ONLINE:
 		return CDS_CLASS (store)->get_folder_info_online (store, top, flags, ex);
@@ -223,8 +278,15 @@ disco_get_folder_info (CamelStore *store, const char *top,
 CamelDiscoStoreStatus
 camel_disco_store_status (CamelDiscoStore *store)
 {
+	CamelService *service = CAMEL_SERVICE (store);
+
 	g_return_val_if_fail (CAMEL_IS_DISCO_STORE (store), CAMEL_DISCO_STORE_ONLINE);
 
+	if (service->status == CAMEL_SERVICE_CONNECTING &&
+	    store->status == CAMEL_DISCO_STORE_ONLINE &&
+	    !camel_session_is_online (service->session))
+		store->status = CAMEL_DISCO_STORE_OFFLINE;
+
 	return store->status;
 }
 
