@@ -9,8 +9,12 @@
 /* Define to terminate lines with "\n" instead of "\r\n" */
 #undef ICAL_UNIX_NEWLINE
 
-/* Define if your libc defines a "timezone" variable
+/* Define if your libc defines a "timezone" variable */
 #undef HAVE_TIMEZONE
-
-/* Define if your libc defines a struct tm containing a "tm_gmtoff" member */
+ 
+/* Define if your libc defines a struct tm containing a "tm_gmtoff" member */  
 #undef HAVE_TM_GMTOFF
+
+/* Define to 1 if your compile does not like lines like: struct
+   something foo[]*/
+#undef ICAL_NO_EMPTY_ARRAY_DECL
