@@ -419,6 +419,12 @@ mail_lookup_handler (const char *mime_type)
 	if (handler)
 		return handler;
 
+	/* Special case MIME type: application/octet-stream
+	 * The point of this type is that there isn't a handler. 
+	 */
+	if (strcmp (mime_type, "application/octet-stream") == 0)
+		return NULL;
+	
 	/* No. Create a new one and look up application and full type
 	 * handler. If we find a builtin, create the handler and
 	 * register it.
@@ -441,6 +447,7 @@ mail_lookup_handler (const char *mime_type)
 	for (iter = components; iter; iter = iter->next) {
 		if (component_supports (iter->data, mime_type)) {
 			handler->generic = FALSE;
+			handler->is_bonobo = TRUE;
 			handler->builtin = handle_via_bonobo;
 			handler->component = OAF_ServerInfo_duplicate (iter->data);
 			gnome_vfs_mime_component_list_free (components);
@@ -474,6 +481,7 @@ mail_lookup_handler (const char *mime_type)
 	if (handler->component) {
 		handler->generic = TRUE;
 		handler->builtin = handle_via_bonobo;
+		handler->is_bonobo = TRUE;
 		goto reg;
 	}
 
@@ -691,11 +699,8 @@ format_mime_part (CamelMimePart *part, MailDisplay *md,
 	
 	if (CAMEL_IS_MULTIPART (wrapper) &&
 	    camel_multipart_get_number (CAMEL_MULTIPART (wrapper)) == 0) {
-		char *mesg;
 		
-		mesg = e_utf8_from_locale_string (_("Could not parse MIME message. Displaying as source."));
-		mail_error_printf (html, stream, "\n%s\n", mesg);
-		g_free (mesg);
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
 		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
 			handle_text_plain (part, "text/plain", md, html, stream);
 		return TRUE;
@@ -788,7 +793,9 @@ write_text_header (const char *name, const char *value, int flags, GtkHTML *html
 	char *encoded;
 
 	if (value && *value)
-		encoded = e_text_to_html (value, E_TEXT_TO_HTML_CONVERT_NL | E_TEXT_TO_HTML_CONVERT_URLS);
+		encoded = e_text_to_html (value, E_TEXT_TO_HTML_CONVERT_NL |
+					  E_TEXT_TO_HTML_CONVERT_SPACES |
+					  E_TEXT_TO_HTML_CONVERT_URLS);
 	else
 		encoded = "";
 	
@@ -1256,7 +1263,7 @@ handle_text_plain (CamelMimePart *part, const char *mime_type,
 	 * has decided to call text/plain because it starts with English
 	 * text...)
 	 */
-	check_specials = !g_strcasecmp (mime_type, "text/plain");
+	check_specials = header_content_type_is (type, "text", "plain");
 	
 	p = text;
 	while (p && check_specials) {
@@ -1432,9 +1439,9 @@ try_uudecoding (char *start, CamelMimePart *mime_part,
 		guint offset, MailDisplay *md, GtkHTML *html, GtkHTMLStream *stream)
 {
 	int mode, len, state = CAMEL_UUDECODE_STATE_INIT;
-	char *filename, *estart, *p, *out;
-	guint32 save = 0;
+	char *filename, *eoln, *p, *out;
 	CamelMimePart *part;
+	guint32 save = 0;
 	
 	/* Make sure it's a real uudecode begin line:
 	 * begin [0-7]+ .*
@@ -1442,29 +1449,35 @@ try_uudecoding (char *start, CamelMimePart *mime_part,
 	mode = strtoul (start + 6, &p, 8);
 	if (p == start + 6 || *p != ' ')
 		return start;
-	estart = strchr (start, '\n');
-	if (!estart)
+	
+	if (!(eoln = strchr (start, '\n')))
 		return start;
 	
-	while (isspace ((unsigned char)*p))
+	while (*p == ' ' || *p == '\t')
 		p++;
-	filename = g_strndup (p, estart++ - p);
+	
+	if (p == eoln)
+		return start;
+	
+	filename = g_strndup (p, eoln - p);
 	
 	/* Make sure there's an end line. */
-	p = strstr (p, "\nend\n");
-	if (!p) {
+	if (!(p = strstr (p, "\nend\n"))) {
 		g_free (filename);
 		return start;
 	}
 	
-	out = g_malloc (p - estart);
-	len = uudecode_step (estart, p - estart, out, &state, &save);
+	eoln++;
+	out = g_malloc (p - eoln);
+	len = uudecode_step (eoln, p - eoln, out, &state, &save);
 	
 	part = fake_mime_part_from_data (out, len, "application/octet-stream",
 					 offset, md);
 	g_free (out);
+	
 	camel_mime_part_set_filename (part, filename);
 	g_free (filename);
+	
 	camel_object_hook_event (CAMEL_OBJECT (md->current_message),
 				 "finalize", destroy_part, part);
 	
@@ -1723,7 +1736,13 @@ handle_multipart_mixed (CamelMimePart *part, const char *mime_type,
 	int i, nparts;
 	gboolean output = FALSE;
 	
-	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
+	if (!CAMEL_IS_MULTIPART (wrapper)) {
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
+		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
+			handle_text_plain (part, "text/plain", md, html, stream);
+		return TRUE;
+	}
+
 	mp = CAMEL_MULTIPART (wrapper);
 
 	nparts = camel_multipart_get_number (mp);	
@@ -1759,7 +1778,7 @@ handle_multipart_encrypted (CamelMimePart *part, const char *mime_type,
 	mpe = CAMEL_MULTIPART_ENCRYPTED (wrapper);
 	
 	camel_exception_init (&ex);
-	cipher = mail_crypto_get_pgp_cipher_context (NULL);
+	cipher = camel_gpg_context_new (session);
 	mime_part = camel_multipart_encrypted_decrypt (mpe, cipher, &ex);
 	camel_object_unref (cipher);
 	
@@ -1792,8 +1811,13 @@ handle_multipart_signed (CamelMimePart *part, const char *mime_type,
 	
 	wrapper = camel_medium_get_content_object (CAMEL_MEDIUM (part));
 	
-	g_return_val_if_fail (CAMEL_IS_MULTIPART_SIGNED (wrapper), FALSE);
-	
+	if (!CAMEL_IS_MULTIPART_SIGNED (wrapper)) {
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
+		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
+			handle_text_plain (part, "text/plain", md, html, stream);
+		return TRUE;
+	}
+
 	mps = CAMEL_MULTIPART_SIGNED (wrapper);
 	
 	/* if subpart & signature is null, what do we do?  just write it out raw?
@@ -1847,7 +1871,7 @@ handle_multipart_signed (CamelMimePart *part, const char *mime_type,
 		/* Write out the verification results */
 		/* TODO: use the right context for the right message ... */
 		camel_exception_init (&ex);
-		cipher = camel_gpg_context_new (session, mail_config_get_pgp_path ());
+		cipher = camel_gpg_context_new (session);
 		if (cipher) {
 			valid = camel_multipart_signed_verify (mps, cipher, &ex);
 			camel_object_unref (CAMEL_OBJECT (cipher));
@@ -1907,8 +1931,13 @@ handle_multipart_related (CamelMimePart *part, const char *mime_type,
 	GHashTable *related_save;
 	int ret;
 	
-	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
-	
+	if (!CAMEL_IS_MULTIPART (wrapper)) {
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
+		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
+			handle_text_plain (part, "text/plain", md, html, stream);
+		return TRUE;
+	}
+
 	mp = CAMEL_MULTIPART (wrapper);
 	nparts = camel_multipart_get_number (mp);	
 	
@@ -2027,7 +2056,12 @@ handle_multipart_alternative (CamelMimePart *part, const char *mime_type,
 	CamelMultipart *multipart;
 	CamelMimePart *mime_part;
 	
-	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
+	if (!CAMEL_IS_MULTIPART (wrapper)) {
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
+		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
+			handle_text_plain (part, "text/plain", md, html, stream);
+		return TRUE;
+	}
 	
 	multipart = CAMEL_MULTIPART (wrapper);
 	
@@ -2047,8 +2081,13 @@ handle_multipart_appledouble (CamelMimePart *part, const char *mime_type,
 		camel_medium_get_content_object (CAMEL_MEDIUM (part));
 	CamelMultipart *multipart;
 
-	g_return_val_if_fail (CAMEL_IS_MULTIPART (wrapper), FALSE);
-	
+	if (!CAMEL_IS_MULTIPART (wrapper)) {
+		mail_error_printf (html, stream, "\n%s\n", U_("Could not parse MIME message. Displaying as source."));
+		if (mail_content_loaded (wrapper, md, TRUE, NULL, html, NULL))
+			handle_text_plain (part, "text/plain", md, html, stream);
+		return TRUE;
+	}
+
 	multipart = CAMEL_MULTIPART (wrapper);
 
 	/* The first part is application/applefile and is not useful
@@ -2375,7 +2414,7 @@ mail_get_message_body (CamelDataWrapper *data, gboolean want_plain, gboolean cit
 	mp = CAMEL_MULTIPART (data);
 	
 	if (CAMEL_IS_MULTIPART_ENCRYPTED (mp)) {
-		cipher = camel_gpg_context_new (session, mail_config_get_pgp_path ());
+		cipher = camel_gpg_context_new (session);
 		subpart = camel_multipart_encrypted_decrypt (CAMEL_MULTIPART_ENCRYPTED (mp),
 							     cipher, NULL);
 		if (!subpart)
