@@ -37,6 +37,7 @@
 #include "e-summary.h"
 #include "e-summary-shown.h"
 #include "e-summary-weather.h"
+#include "e-summary-preferences.h"
 #include "weather.h"
 #include "metar.h"
 
@@ -320,7 +321,6 @@ e_summary_weather_update (ESummary *summary)
 
 		uri = g_strdup_printf ("http://weather.noaa.gov/cgi-bin/mgetmetar.pl?cccc=%s", weather->location);
 		context = soup_context_get (uri);
-		g_print ("Updating %s\n", uri);
 		if (context == NULL) {
 			g_warning ("Invalid URL: %s", uri);
 			soup_context_unref (context);
@@ -759,6 +759,7 @@ e_summary_weather_reconfigure (ESummary *summary)
 {
 	ESummaryWeather *weather;
 	GList *old, *p;
+	GList *weather_list;
 
 	g_return_if_fail (summary != NULL);
 	g_return_if_fail (IS_E_SUMMARY (summary));
@@ -768,14 +769,20 @@ e_summary_weather_reconfigure (ESummary *summary)
 	/* Stop timeout so it doesn't occur while we're changing stuff*/
 	gtk_timeout_remove (weather->timeout);
 
-	for (old = weather->weathers; old; old = old->next) {
+	/* Clear the weather list before doing weather_free() on each of them
+	   because otherwise soup_message_cancel() could invoke the refresh
+	   function just while we are freeing things.  [#31639]  */
+	weather_list = weather->weathers;
+	weather->weathers = NULL;
+
+	for (old = weather_list; old != NULL; old = old->next) {
 		Weather *w;
 
 		w = old->data;
 		weather_free (w);
 	}
-	g_list_free (weather->weathers);
-	weather->weathers = NULL;
+	g_list_free (weather_list);
+
 	for (p = summary->preferences->stations; p; p = p->next) {
 		e_summary_weather_add_location (summary, p->data);
 	}
