@@ -4,9 +4,8 @@
  * Copyright (C) 2000, 2001 Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -333,6 +332,7 @@ e_splash_construct (ESplash *splash,
 	gtk_window_set_position (GTK_WINDOW (splash), GTK_WIN_POS_CENTER);
 	gtk_window_set_policy (GTK_WINDOW (splash), FALSE, FALSE, FALSE);
 	gtk_window_set_default_size (GTK_WINDOW (splash), image_width, image_height);
+	gtk_window_set_wmclass (GTK_WINDOW (splash), "evolution-splash", "Evolution");
 	gnome_window_icon_set_from_file (GTK_WINDOW (splash), EVOLUTION_DATADIR "/pixmaps/evolution.png");
 	gtk_window_set_title (GTK_WINDOW (splash), "Ximian Evolution");
 
@@ -418,6 +418,7 @@ e_splash_set_icon_highlight  (ESplash *splash,
 	priv = splash->priv;
 
 	icon = (Icon *) g_list_nth_data (priv->icons, num);
+	g_return_if_fail (icon != NULL);
 
 	gtk_object_set (GTK_OBJECT (icon->canvas_item),
 			"pixbuf", highlight ? icon->light_pixbuf : icon->dark_pixbuf,
