@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +23,7 @@
 #ifndef EVOLUTION_IMPORTER_LISTENER_H
 #define EVOLUTION_IMPORTER_LISTENER_H
 
+#include <glib.h>
 #include <bonobo/bonobo-xobject.h>
 #include <importer/GNOME_Evolution_Importer.h>
 #include "evolution-importer.h"
